/*
 * IFileProcessor.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Authors: WishingBone, TCSDEVELOPER
 */

using System;
using System.IO;
using System.Collections;
using TopCoder.File.Statistics.Reporting;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// The extended version of <see cref="FileStatistics"/> which supports some advanced operations
    /// including directory processing and report printing.
    /// </summary>
    ///
    /// <remarks>
    /// <para>
    /// The reason to separate this out is that we may not need to rely on the directory traversing
    /// algorithm incorporated here (which is recursive).
    /// </para>
    /// <para>
    /// This class shares the same configuration logic as <see cref="FileStatistics"/>.
    /// </para>
    /// </remarks>
    ///
    /// <threadsafety static="false" instance="false"/>
    public class ExtendedFileStatistics : FileStatistics
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="ExtendedFileStatistics"/> class.
        /// </summary>
        public ExtendedFileStatistics()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ExtendedFileStatistics"/> class
        /// using configuration values from the specified nameSpace.
        /// </summary>
        ///
        /// <remarks>
        /// The namespace should be already loaded.
        /// It loads the configured processors, the reporting, as well as the matchall flag.
        /// </remarks>
        ///
        /// <param name="nameSpace">
        /// The namespace to load configuration values from.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="nameSpace"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="nameSpace"/> is empty after trimmed.
        /// </exception>
        /// <exception cref="ConfigurationException">
        /// A config error occurs.
        /// </exception>
        public ExtendedFileStatistics(string nameSpace)
            : base(nameSpace)
        {
        }

        /// <summary>
        /// Processes a directory in a non-recursive mode.
        /// </summary>
        ///
        /// <param name="directory">
        /// The directory to process.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="directory"/> is a null reference.
        /// </exception>
        /// <exception cref="IOException">
        /// An IO operation fails.
        /// </exception>
        /// <exception cref="FileStatisticsException">
        /// A processor fails.
        /// </exception>
        public void ProcessDirectory(DirectoryInfo directory)
        {
            // ANE is delegated as well
            ProcessDirectory(directory, false);
        }

        /// <summary>
        /// Processes a directory in the given mode.
        /// </summary>
        ///
        /// <remarks>
        /// If it's non-recursive, only the files in the directory is processed;
        /// otherwise every files in sub-directories are processed.
        /// </remarks>
        ///
        /// <param name="directory">
        /// The directory to process.
        /// </param>
        /// <param name="recursive">
        /// Whether to process recursively.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="directory"/> is a null reference.
        /// </exception>
        /// <exception cref="IOException">
        /// An IO operation fails.
        /// </exception>
        /// <exception cref="FileStatisticsException">
        /// A processor fails.
        /// </exception>
        public void ProcessDirectory(DirectoryInfo directory, bool recursive)
        {
            // cannot be null
            if (directory == null)
            {
                throw new ArgumentNullException("directory");
            }

            // process all files in this dir first
            foreach (FileInfo files in directory.GetFiles())
            {
                ProcessFile(files);
            }

            if (recursive)
            {
                // process sub dirs
                foreach (DirectoryInfo dirs in directory.GetDirectories())
                {
                    ProcessDirectory(dirs, recursive);
                }
            }
        }

        /// <summary>
        /// Prints the report to console.
        /// </summary>
        ///
        /// <remarks>
        /// The report will be printed to the console
        /// using the specified <see cref="IStatsReporting.GenerateReport"/> property.
        /// </remarks>
        ///
        /// <exception cref="StatsReportingException">
        /// An error occurs.
        /// </exception>
        public void PrintReport()
        {
            Console.WriteLine(Reporting.GenerateReport());
        }

        /// <summary>
        /// Prints the report to given file.
        /// </summary>
        ///
        /// <remarks>
        /// The report will be saved to the file
        /// using the specified <see cref="IStatsReporting.GenerateReport"/> property.
        /// If the file already exists, it'll be overwritten.
        /// </remarks>
        ///
        /// <param name="file">
        /// The file to write to.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="file"/> is a null reference.
        /// </exception>
        /// <exception cref="IOException">
        /// An IO operation fails.
        /// </exception>
        /// <exception cref="StatsReportingException">
        /// If rendering fails.
        /// </exception>
        public void PrintReport(FileInfo file)
        {
            // cannot be null
            if (file == null)
            {
                throw new ArgumentNullException("file");
            }

            try
            {
                // create the file and write the report
                using (StreamWriter writer = file.CreateText())
                {
                    writer.Write(Reporting.GenerateReport());
                }
            }
            catch (IOException)
            {
                // just re-throw
                throw;
            }
            catch (StatsReportingException)
            {
                // just re-throw
                throw;
            }
            catch (Exception e)
            {
                throw new StatsReportingException(string.Format(
                    "An error occuring printing report to file {0}", file.FullName), e);
            }
        }
    }
}
