/*
 * AnyFileMatcher.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Authors: WishingBone, TCSDEVELOPER
 */

using System;
using System.IO;

namespace TopCoder.File.Statistics.Matcher
{
    /// <summary>
    /// <see cref="AnyFileMatcher"/> is a trivial implementation of <see cref="IFileMatcher"/>
    /// that matches any file.
    /// </summary>
    ///
    /// <remarks>
    /// This implementation does not need any configuration.
    /// The constructor with nameSpace and prefix arguments is provided
    /// for reflection purpose only.
    /// </remarks>
    ///
    /// <threadsafety static="true" instance="true"/>
    public class AnyFileMatcher : IFileMatcher
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="AnyFileMatcher"/> instance.
        /// </summary>
        public AnyFileMatcher()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="AnyFileMatcher"/> instance
        /// with the specified namespace and prefix.
        /// </summary>
        ///
        /// <remarks>
        /// This constructor is provided for reflection purpose only.
        /// No work is actually performed by this constructor as
        /// no configuration is needed by this class.
        /// </remarks>
        ///
        /// <param name="nameSpace">
        /// Not applicable.
        /// </param>
        /// <param name="prefix">
        /// Not applicable.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// Any of the argument is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="nameSpace"/> is empty.
        /// </exception>
        public AnyFileMatcher(string nameSpace, string prefix)
        {
            // cannot be null
            if (nameSpace == null)
            {
                throw new ArgumentNullException("nameSpace");
            }
            if (prefix == null)
            {
                throw new ArgumentNullException("prefix");
            }
            // nameSpace cannot be empty
            if (nameSpace.Trim().Length == 0)
            {
                throw new ArgumentException("'nameSpace' cannot be empty", "nameSpace");
            }
        }

        /// <summary>
        /// Always returns <b>true</b> for any file,
        /// even if the file doesn't exist.
        /// </summary>
        ///
        /// <param name="file">
        /// The file to match.
        /// </param>
        ///
        /// <returns>
        /// Always <b>true</b>.
        /// </returns>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="file"/> is a null reference.
        /// </exception>
        public bool Match(FileInfo file)
        {
            // cannot be null
            if (file == null)
            {
                throw new ArgumentNullException("file");
            }

            // always returns true
            return true;
        }
    }
}
