/*
 * TestBinaryFileMatcher.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using System.IO;
using NUnit.Framework;
using TopCoder.File.Statistics.Tests;

namespace TopCoder.File.Statistics.Matcher.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="BinaryFileMatcher"/> class.
    /// </summary>
    [TestFixture]
    public class TestBinaryFileMatcher : TestHelper
    {
        /// <summary>
        /// The <see cref="BinaryFileMatcher"/> instance to test.
        /// Initialized in SetUp().
        /// </summary>
        private IFileMatcher matcher;

        /// <summary>
        /// Initializes the <see cref="BinaryFileMatcher"/> instance to test.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            matcher = new BinaryFileMatcher();
        }

        /// <summary>
        /// Tests constructor() doesn't throw any exception.
        /// </summary>
        [Test]
        public void TestConstructor1()
        {
            new BinaryFileMatcher();
        }

        /// <summary>
        /// Tests constructor(string, string) doesn't throw any exception
        /// given valid nameSpace and prefix.
        /// </summary>
        [Test]
        public void TestConstructor2()
        {
            // this really does nothing
            new BinaryFileMatcher(ValidNamespace, "binary.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ArgumentNullException
        /// given null nameSpace.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructor2NullNamespace()
        {
            new BinaryFileMatcher(null, "binary.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ArgumentNullException
        /// given null prefix.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructor2NullPrefix()
        {
            new BinaryFileMatcher(ValidNamespace, null);
        }

        /// <summary>
        /// Tests constructor(string, string) throws ArgumentException
        /// given empty namespace.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructor2EmptyNamespace()
        {
            new BinaryFileMatcher(" ", "binary.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws nothing
        /// given non-existing prefix because no configuration is needed.
        /// </summary>
        [Test]
        public void TestConstructor2NotExist()
        {
            new BinaryFileMatcher(ValidNamespace, "notexists.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws nothing
        /// given non-existing empty prefix.
        /// </summary>
        [Test]
        public void TestConstructor2NotExistEmpty()
        {
            new BinaryFileMatcher(ValidNamespace, "");
        }

        /// <summary>
        /// Tests constructor(string, string) throws nothing
        /// given non-existing empty-space prefix.
        /// </summary>
        [Test]
        public void TestConstructor2NotExistEmptySpace()
        {
            new BinaryFileMatcher(ValidNamespace, " ");
        }

        /// <summary>
        /// Tests Match(FileInfo) with a binary file.
        /// </summary>
        [Test]
        public void TestMatchBinary()
        {
            // a binary file
            Assert.IsTrue(matcher.Match(new FileInfo(@"..\..\test_files\Matcher_FileCategory_Binary.bin")),
                "The file should be detected as a binary");
        }

        /// <summary>
        /// Tests Match(FileInfo) with a text file.
        /// </summary>
        [Test]
        public void TestMatchText()
        {
            // a text file
            Assert.IsFalse(matcher.Match(new FileInfo(@"..\..\test_files\Matcher_FileCategory_Text.txt")),
                "The file should be detected as a text");
        }

        /// <summary>
        /// Tests Match() throws IOException
        /// given a non-existing file.
        /// </summary>
        [Test]
        public void TestMatchNotExist()
        {
            try
            {
                // the file doesn't exist
                // this may throw FileNotFoundException, which is a subclass of IOException
                matcher.Match(new FileInfo(@"..\..\test_files\notexists.txt"));
                Assert.Fail("IOException expected");
            }
            catch (IOException)
            {
                // ok
            }
        }

        /// <summary>
        /// Tests Match() throws IOException
        /// given an existing directory.
        /// </summary>
        [Test]
        [ExpectedException(typeof(IOException))]
        public void TestMatchDirectory()
        {
            // an existing directory
            matcher.Match(new FileInfo(@"..\..\test_files"));
        }

        /// <summary>
        /// Tests Match() throws ArgumentNullException
        /// given null FileInfo.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestMatchNull()
        {
            matcher.Match(null);
        }
    }
}
