/*
 * TestTypeNameMatcher.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using System.Collections;
using System.IO;
using NUnit.Framework;
using TopCoder.File.Statistics.Tests;

namespace TopCoder.File.Statistics.Matcher.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="TypeNameMatcher"/> class.
    /// </summary>
    [TestFixture]
    public class TestTypeNameMatcher : TestHelper
    {
        /// <summary>
        /// The <see cref="TypeNameMatcher"/> instance to test.
        /// Initialized in each unit test case according to its need.
        /// </summary>
        private IFileMatcher matcher;

        /// <summary>
        /// A list of type names to match.
        /// Initialized in SetUp() and used to test constructor(IList).
        /// </summary>
        private IList types;

        /// <summary>
        /// Initializes the <see cref="TypeNameMatcher"/> instance to test
        /// using the test_files/Matcher_TypeName.xml.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            // load Magic Number configuration
            // load test_files/Matcher_TypeName.xml which is
            // used for constructor with nameSpace and prefix argument
            base.LoadConfigFile(MagicNumberConfig, @"..\..\test_files\Matcher_TypeName.xml");

            // initialize type names list (with "Java class file" and "XML file")
            types = new ArrayList();
            types.Add("Java class file");
            types.Add("XML file");
        }

        /// <summary>
        /// Tears down configuration manager.
        /// </summary>
        [TearDown]
        public void TearDown()
        {
            manager.Clear();
        }

        /// <summary>
        /// Tests the static property key name.
        /// </summary>
        [Test]
        public void TestStaticPropertyKey()
        {
            Assert.AreEqual("types", TypeNameMatcher.TypesPropertyKey,
                "The property key name is wrong");
        }

        /// <summary>
        /// Tests constructor(string) is initialized with a single file type name
        /// and Match() only returns true for the specified type.
        /// </summary>
        [Test]
        public void TestConstructor1()
        {
            // XML file
            matcher = new TypeNameMatcher("XML file");

            // txt, java class file should not match
            Assert.IsFalse(matcher.Match(new FileInfo(SampleText)),
                "txt should not match");
            Assert.IsFalse(matcher.Match(new FileInfo(@"..\..\test_files\Matcher_FileType_MagicNumber.class")),
                "java class file should not match");

            // xml file should match
            Assert.IsTrue(matcher.Match(new FileInfo(@"..\..\test_files\FileStatistics.xml")),
                "XML file should match");
        }

        /// <summary>
        /// Tests constructor(string) is initialized with a file type name that is
        /// wrong in case and doesn't match that defined in Magic Number.
        /// </summary>
        [Test]
        public void TestConstructor1Case()
        {
            // xml file (wrong case)
            matcher = new TypeNameMatcher("xml file");

            // txt, java class file should not match
            Assert.IsFalse(matcher.Match(new FileInfo(SampleText)),
                "txt should not match");
            Assert.IsFalse(matcher.Match(new FileInfo(@"..\..\test_files\Matcher_FileType_MagicNumber.class")),
                "java class file should not match");

            // xml file should not match either
            Assert.IsFalse(matcher.Match(new FileInfo(@"..\..\test_files\FileStatistics.xml")),
                "XML file should not match because the case is wrong");
        }

        /// <summary>
        /// Tests constructor(string) throws ArgumentNullException
        /// given null argument.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructor1Null()
        {
            new TypeNameMatcher((string) null);
        }

        /// <summary>
        /// Tests constructor(string) throws ArgumentException
        /// given empty argument.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructor1Empty()
        {
            new TypeNameMatcher(" ");
        }

        /// <summary>
        /// Tests constructor(IList) is initialized with two file types
        /// and Match() only returns true for the specified type.
        /// </summary>
        [Test]
        public void TestConstructor2()
        {
            // Java class file and XML file
            matcher = new TypeNameMatcher(types);

            // txt should not match
            Assert.IsFalse(matcher.Match(new FileInfo(SampleText)),
                "txt should not match");

            // java class file, xml file should match
            Assert.IsTrue(matcher.Match(new FileInfo(@"..\..\test_files\Matcher_FileType_MagicNumber.class")),
                "Java class file should match");
            Assert.IsTrue(matcher.Match(new FileInfo(@"..\..\test_files\FileStatistics.xml")),
                "XML file should match");
        }

        /// <summary>
        /// Tests constructor(IList) is initialized with a file type that is wrong in case
        /// and doesn't match that defined in Magic Number.
        /// </summary>
        [Test]
        public void TestConstructor2Case()
        {
            // Magic number defines "Java class file"
            types.Clear();
            types.Add("java class file");
            matcher = new TypeNameMatcher(types);

            // all should not match
            Assert.IsFalse(matcher.Match(new FileInfo(SampleText)),
                "txt should not match");
            Assert.IsFalse(matcher.Match(new FileInfo(@"..\..\test_files\Matcher_FileType_MagicNumber.class")),
                "Java class file should not match");
            Assert.IsFalse(matcher.Match(new FileInfo(@"..\..\test_files\FileStatistics.xml")),
                "XML file should not match");
        }

        /// <summary>
        /// Tests constructor(IList) throws ArgumentNullException
        /// given null argument.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructor2Null()
        {
            new TypeNameMatcher((IList) null);
        }

        /// <summary>
        /// Tests constructor(IList) throws ArgumentException
        /// given empty list.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructor2List()
        {
            new TypeNameMatcher(new ArrayList());
        }

        /// <summary>
        /// Tests constructor(IList) throws ArgumentException
        /// given list containing null element.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructor2NullElement()
        {
            types.Add(null);
            new TypeNameMatcher(types);
        }

        /// <summary>
        /// Tests constructor(IList) throws ArgumentException
        /// given list containing non-string element.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructor2NonStringElement()
        {
            types.Add(123);
            new TypeNameMatcher(types);
        }

        /// <summary>
        /// Tests constructor(IList) throws ArgumentException
        /// given list containing empty string element.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructor2EmptyStringElement()
        {
            types.Add(" ");
            new TypeNameMatcher(types);
        }

        /// <summary>
        /// Tests constructor(IList) throws ArgumentException
        /// given list containing duplicate element.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructor2DuplicateElement()
        {
            types.Add(types[0]);
            new TypeNameMatcher(types);
        }

        /// <summary>
        /// Tests constructor(string, string) is initialized with two file type names
        /// and Match() only returns true for the specified types.
        /// </summary>
        [Test]
        public void TestConstructor3()
        {
            // initialize TypeNameMatcher ("Java class file" and "XML file")
            matcher = new TypeNameMatcher(ValidNamespace, "matcher.");

            // txt should not match
            Assert.IsFalse(matcher.Match(new FileInfo(SampleText)),
                "txt should not match");

            // java class file, xml file should match
            Assert.IsTrue(matcher.Match(new FileInfo(@"..\..\test_files\Matcher_FileType_MagicNumber.class")),
                "Java class file should match");
            Assert.IsTrue(matcher.Match(new FileInfo(@"..\..\test_files\FileStatistics.xml")),
                "XML file should match");
        }

        /// <summary>
        /// Tests constructor(string, string) is initialized with a file type that is wrong in case
        /// and doesn't match that defined in Magic Number.
        /// </summary>
        [Test]
        public void TestConstructor3Case()
        {
            // initialize TypeNameMatcher ("java class file")
            matcher = new TypeNameMatcher(ValidNamespace, "matcher_incorrect.");

            // all should not match
            Assert.IsFalse(matcher.Match(new FileInfo(SampleText)),
                "txt should not match");
            Assert.IsFalse(matcher.Match(new FileInfo(@"..\..\test_files\Matcher_FileType_MagicNumber.class")),
                "Java class file should not match");
            Assert.IsFalse(matcher.Match(new FileInfo(@"..\..\test_files\FileStatistics.xml")),
                "XML file should not match");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ArgumentNullException
        /// given null nameSpace.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructor3NullNamespace()
        {
            new TypeNameMatcher(null, "matcher.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ArgumentNullException
        /// given null prefix.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructor3NullPrefix()
        {
            new TypeNameMatcher(ValidNamespace, null);
        }

        /// <summary>
        /// Tests constructor(string, string) throws ArgumentException
        /// given empty namespace.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructor3EmptyNamespace()
        {
            new TypeNameMatcher(" ", "matcher.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given non-existing prefix.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructor3NotExist()
        {
            new TypeNameMatcher(ValidNamespace, "notexists.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given non-existing empty prefix.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructor3NotExistEmpty()
        {
            new TypeNameMatcher(ValidNamespace, "");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given non-existing empty-space prefix.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructor3NotExistEmptySpace()
        {
            new TypeNameMatcher(ValidNamespace, " ");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given no list.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructor3NoList()
        {
            new TypeNameMatcher(ValidNamespace, "matcher_invalid.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given empty string file type name.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructor3EmptyString()
        {
            new TypeNameMatcher(ValidNamespace, "matcher_empty.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given list containing duplicate element.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructor3DuplicateElement()
        {
            new TypeNameMatcher(ValidNamespace, "matcher_duplicate.");
        }

        /// <summary>
        /// Tests Match() throws ArgumentNullException
        /// given null FileInfo.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestMatchNull()
        {
            matcher.Match(null);
        }
    }
}
