/*
 * TestBasicStatsReporting.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using System.Collections;
using System.IO;
using NUnit.Framework;
using TopCoder.File.Statistics.Matcher;
using TopCoder.File.Statistics.Processor.LineCounter;
using TopCoder.File.Statistics.Tests;

namespace TopCoder.File.Statistics.Reporting.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="BasicStatsReporting"/> class.
    /// </summary>
    [TestFixture]
    public class TestBasicStatsReporting : TestHelper
    {
        /// <summary>
        /// A sample Metric name.
        /// </summary>
        private const string name = "Word count";

        /// <summary>
        /// A sample Metric display name.
        /// </summary>
        private const string displayName = "wcount";

        /// <summary>
        /// A sample Metric description.
        /// </summary>
        private const string description = "Count the number of words";

        /// <summary>
        /// The processor alias name.
        /// We'll test with 2 processor alias, composed of alias + "1"/"2".
        /// See SetUp().
        /// </summary>
        private const string alias = "Processor alias";

        /// <summary>
        /// A sample initial metric to be stored in StatsResult.
        /// Initialized in SetUp().
        /// </summary>
        private Metric metric;

        /// <summary>
        /// A sample statistic result for a file.
        /// Initialized in SetUp().
        /// </summary>
        private StatsResult result;

        /// <summary>
        /// The IFileProcessor which generates the result.
        /// Only the alias name is important for the report.
        /// We'll create 2 processor instances with the name alias + "1"/"2".
        /// Initialized to TextLineCounter() in SetUp().
        /// </summary>
        private IFileProcessor[] processor;

        /// <summary>
        /// The <see cref="BasicStatsReporting"/> instance to test.
        /// Initialized in SetUp() with 1 sample result.
        /// </summary>
        private IStatsReporting report;

        /// <summary>
        /// Initializes the <see cref="BasicStatsReporting"/> instance to test.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            // initialize the IFileProcessor
            // only the alias name is important
            processor = new IFileProcessor[]
                {
                    new TextLineCounter(alias + "1", new AnyFileMatcher()),
                    new TextLineCounter(alias + "2", new AnyFileMatcher())
                };

            // create a StatsResult containing a metric with an initial value of 10
            metric = new Metric(name, displayName, description);
            result = new StatsResult();
            result[metric] = 10;

            // initialize the BasicStatsReporting
            // reset to test_files folder
            // and aggregate a result
            report = new BasicStatsReporting();
            report.Reset(new DirectoryInfo(@"..\..\test_files"));
            report.Aggregate(processor[0], new FileInfo(SampleText), result);
        }

        /// <summary>
        /// Tests GenerateReport.
        /// </summary>
        [Test]
        public void TestGenerateReport()
        {
            // generate the report and load the expected result
            string rawOutput = report.GenerateReport();
            string expectedOutput = ReadContent(@"..\..\test_files\Reporting_Result_Basic.txt", false);

            // remove the header, because it contains system specific path
            int pos1 = rawOutput.IndexOf('=');
            int pos2 = expectedOutput.IndexOf('=');
            Assert.AreEqual(expectedOutput.Substring(pos2),
                rawOutput.Substring(pos1),
                "The basic output format is not as expected");
        }

        /// <summary>
        /// Tests GenerateReport given another statistic
        /// with different metric.
        /// </summary>
        [Test]
        public void TestGenerateReportAdd()
        {
            // create a new Metric and add to existing result
            result[new Metric(name + "1", displayName + "1", description + "1")] = 7;
            // aggregate for a new file with processor[1]
            report.Aggregate(processor[1], new FileInfo(@"..\..\test_files\test\aaa\sample.txt"), result);

            // generate the report and load the expected result
            string rawOutput = report.GenerateReport();
            string expectedOutput = ReadContent(@"..\..\test_files\Reporting_Result_Basic_Two.txt", false);

            // remove the header, because it contains system specific path
            int pos1 = rawOutput.IndexOf('=');
            int pos2 = expectedOutput.IndexOf('=');
            Assert.AreEqual(expectedOutput.Substring(pos2),
                rawOutput.Substring(pos1),
                "The basic output format is not as expected");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ArgumentNullException
        /// given null nameSpace.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructor2NullNamespace()
        {
            new BasicStatsReporting(null, "reporting.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ArgumentNullException
        /// given null prefix.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructor3NullPrefix()
        {
            new BasicStatsReporting(ValidNamespace, null);
        }

        /// <summary>
        /// Tests constructor(string, string) throws ArgumentException
        /// given empty namespace.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructor3EmptyNamespace()
        {
            new BasicStatsReporting(" ", "reporting.");
        }
    }
}
