/*
 * FileTypeMatcher.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Authors: WishingBone, TCSDEVELOPER
 */

using System;
using System.IO;
using TopCoder.Unix.MagicNumbers;

namespace TopCoder.File.Statistics.Matcher
{
    /// <summary>
    /// <see cref="FileTypeMatcher"/> is a template class
    /// that handles file type oriented matching logic.
    /// </summary>
    ///
    /// <remarks>
    /// File type is resolved by the Magic Numbers component.
    /// The MagicNumber class will be instantiated with the default namespace
    /// of <c>TopCoder.Unix.MagicNumbers.MagicNumber</c>.
    /// Please refer to the Magic Number component for its proper configuration.
    /// </remarks>
    ///
    /// <threadsafety static="true" instance="true"/>
    public abstract class FileTypeMatcher : IFileMatcher
    {
        /// <summary>
        /// A cache of the MagicNumber instance.
        /// This is initialized on the first invocation of Match().
        /// Subsequent calls on Match() will just reuse this.
        /// </summary>
        private MagicNumber magic = null;

        /// <summary>
        /// Initializes a new instance of the <see cref="FileTypeMatcher"/> instance.
        /// </summary>
        protected FileTypeMatcher()
        {
        }

        /// <summary>
        /// Obtains the file type from the Magic Numbers component and
        /// uses the template method to decide whether the file matches.
        /// </summary>
        ///
        /// <remarks>
        /// Please refer to the Magic Numbers documentation for its proper configuration.
        /// </remarks>
        ///
        /// <param name="file">
        /// The file to match.
        /// </param>
        ///
        /// <returns>
        /// <b>true</b> if the file type matches; otherwise, <b>false</b>.
        /// </returns>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="file"/> is a null reference.
        /// </exception>
        /// <exception cref="IOException">
        /// An IO operation fails.
        /// </exception>
        /// <exception cref="FileMatchingException">
        /// An error, other than IO error, occurs.
        /// </exception>
        public bool Match(FileInfo file)
        {
            // cannot be null
            if (file == null)
            {
                throw new ArgumentNullException("file");
            }

            try
            {
                // instantiate and cache NagicNumber
                if (magic == null)
                {
                    magic = new MagicNumber();
                }

                // determine file type and call the template method
                // if file type is undetected, just return false
                FileType type = magic.DetermineFileType(file);
                if (type == null)
                {
                    return false;
                }
                else
                {
                    return Match(type);
                }
            }
            catch (UnauthorizedAccessException e)
            {
                // we get this exception if file is a directory
                throw new IOException(string.Format(
                    "IO error for file {0}", file.FullName), e);
            }
            catch (IOException)
            {
                // just re-throw
                throw;
            }
            catch (FileMatchingException)
            {
                // just re-throw
                throw;
            }
            catch (Exception e)
            {
                // encapsulate all other errors in FileMatchingException
                throw new FileMatchingException(string.Format(
                    "Error matching for filename {0}", file.FullName), e);
            }
        }

        /// <summary>
        /// The template method that matches the file with file type lists.
        /// </summary>
        ///
        /// <param name="type">
        /// The file type to match.
        /// </param>
        ///
        /// <returns>
        /// <b>true</b> if the file type matches; otherwise, <b>false</b>.
        /// </returns>
        ///
        /// <exception cref="FileMatchingException">
        /// An error occurs.
        /// </exception>
        protected abstract bool Match(FileType type);
    }
}
