/*
 * TypeNameMatcher.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Authors: WishingBone, TCSDEVELOPER
 */

using System;
using System.Collections;
using TopCoder.Unix.MagicNumbers;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics.Matcher
{
    /// <summary>
    /// Matches file types with one or more type names (as configured in Magic Numbers).
    /// </summary>
    ///
    /// <remarks>
    /// The file type is matched in a case-sensitive manner
    /// against a list of type names.
    /// Please note that the type names should match with those defined in
    /// the Magic Number component (case-sensitive).
    /// It is possible to instantiate this class using reflection
    /// and has its type names list loaded from a configuration file.
    /// The constructor with nameSpace and prefix arguments is used for this purpose.
    /// There should be a multi-value property with the name <c>prefix + "types"</c>,
    /// which contains the type names list.
    /// </remarks>
    ///
    /// <threadsafety static="true" instance="true"/>
    public class TypeNameMatcher : FileTypeMatcher
    {
        /// <summary>
        /// The configuration key that contains multiple file type names to match.
        /// </summary>
        ///
        /// <remarks>
        /// The actual property key name is <c>prefix + "types"</c>.
        /// The matching is carried out in a case-sensitive manner.
        /// The property is required, and empty file type name is not allowed
        /// even if it is valid in the Magic Number component.
        /// </remarks>
        public static readonly string TypesPropertyKey = "types";

        /// <summary>
        /// A list of file name types (string) to match.
        /// This list is initialized in constructor and is immutable.
        /// Developer notes. Although the design recommends IList, it's
        /// better to use IDictionary; because IList.Contains is O(n),
        /// while IDictionary.Contains can be O(1).
        /// It's currently implemented with a HashTable.
        /// </summary>
        private IDictionary types;

        /// <summary>
        /// Initializes a new instance of the <see cref="TypeNameMatcher"/>
        /// with the specified single file type name.
        /// </summary>
        ///
        /// <param name="type">
        /// A single file type name to match.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="type"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="type"/> is empty.
        /// </exception>
        public TypeNameMatcher(string type)
        {
            // cannot be null nor empty
            if (type == null)
            {
                throw new ArgumentNullException("type");
            }
            if (type.Trim().Length == 0)
            {
                throw new ArgumentException("'type' cannot be empty", "type");
            }

            // save to instance members
            this.types = new Hashtable();
            this.types.Add(type, null);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="TypeNameMatcher"/>
        /// with the specified list of file type names.
        /// </summary>
        ///
        /// <param name="types">
        /// The list of file type names (string) to match.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="types"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="types"/> is empty, contains non-string instance,
        /// contains empty string (after trimmed), or contains duplicate element.
        /// </exception>
        public TypeNameMatcher(IList types)
        {
            // cannot be null
            if (types == null)
            {
                throw new ArgumentNullException("types");
            }
            // cannot be empty
            if (types.Count == 0)
            {
                throw new ArgumentException("'types' cannot be an empty list", "types");
            }
            // cannot contain non-string instance
            foreach (object obj in types)
            {
                if (!(obj is string))
                {
                    throw new ArgumentException(
                        "'types' contains a non-string instance", "types");
                }
                // cannot contain empty string
                if (((string) obj).Trim().Length == 0)
                {
                    throw new ArgumentException(
                        "'types' contains empty string", "types");
                }
            }

            // add list to hashtable (duplicate will throw ArgumentException)
            this.types = new Hashtable();
            Helper.AddListToDictionary(types, this.types);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="TypeNameMatcher"/>
        /// which loads the file type names list from the specified nameSpace
        /// and prefix of the configuration.
        /// </summary>
        ///
        /// <remarks>
        /// The actual property key name is <c>prefix + "types"</c>
        /// and multiple values are possible.
        /// </remarks>
        ///
        /// <param name="nameSpace">
        /// The namespace to load configuration values from.
        /// </param>
        /// <param name="prefix">
        /// The prefix of the property name to load configuration values from.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// Any of the argument is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="nameSpace"/> is empty.
        /// </exception>
        /// <exception cref="ConfigurationException">
        /// The property name cannot be found or
        /// it does not hold any values or any value is empty after trimmed
        /// or it contains duplicate values.
        /// </exception>
        public TypeNameMatcher(string nameSpace, string prefix)
        {
            // cannot be null
            if (nameSpace == null)
            {
                throw new ArgumentNullException("nameSpace");
            }
            if (prefix == null)
            {
                throw new ArgumentNullException("prefix");
            }
            // nameSpace cannot be empty
            if (nameSpace.Trim().Length == 0)
            {
                throw new ArgumentException("'nameSpace' cannot be empty", "nameSpace");
            }

            // the property name
            string propName = prefix + TypesPropertyKey;
            try
            {
                // load the multi-value property
                ConfigManager cm = ConfigManager.GetInstance();
                string[] types = cm.GetValues(nameSpace, propName);

                // required and cannot be empty
                if ((types == null) || (types.Length == 0))
                {
                    throw new ConfigurationException(string.Format(
                        "{0} cannot be found or is empty", propName));
                }

                // no element can be empty after trimmed
                foreach (string s in types)
                {
                    if (s.Trim().Length == 0)
                    {
                        throw new ConfigurationException(string.Format(
                            "{0} should not contain empty string", propName));
                    }
                }

                // add list to dictionary
                // (duplicate will throw ArgumentException, encapsulated to ConfigurationException)
                this.types = new Hashtable();
                Helper.AddListToDictionary(types, this.types);
            }
            catch (ConfigurationException)
            {
                // just re-throw
                throw;
            }
            catch (Exception e)
            {
                // encapsulate all others in ConfigurationException
                throw new ConfigurationException(string.Format(
                    "Error initializing from namespace {0}, property {1}", nameSpace, propName), e);
            }
        }

        /// <summary>
        /// Checks whether the file type name matches any of the type names.
        /// </summary>
        ///
        /// <remarks>
        /// The name of the FileType will be compared in a case-sensitive
        /// manner with the registered types.
        /// </remarks>
        ///
        /// <param name="type">
        /// The FileType whose name to match.
        /// </param>
        ///
        /// <returns>
        /// <b>true</b> if the file type name match; otherwise, <b>false</b>
        /// </returns>
        protected override bool Match(FileType type)
        {
            // match against the list (O(1) operation)
            return types.Contains(type.Name);
        }
    }
}
