/*
 * FileProcessorBase.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Authors: WishingBone, TCSDEVELOPER
 */

using System;
using System.IO;
using System.Collections;
using TopCoder.File.Statistics.Matcher;
using TopCoder.File.Statistics.Processor.LineCounter;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics.Processor
{
    /// <summary>
    /// A base implementation of <see cref="IFileProcessor"/> which provides alias and matcher support,
    /// including loading them from configuration file.
    /// </summary>
    ///
    /// <remarks>
    /// <para>
    /// It serves <see cref="TextLineCounter"/> and <see cref="CStyleLineCounter"/>
    /// as well as can be genericly used by future implementations.
    /// </para>
    /// <para>
    /// It is possible to initialize this class using a configuration value
    /// using the constructor that takes the nameSpace and prefix arguments.
    /// The property names must be constructed with the prefix argument.
    /// Two property keys are required under the specified prefix:
    /// <list type="bullet">
    /// <item>
    ///     <term>alias</term>
    ///     <description>
    ///     The alias name for this processor (Required and cannot be empty).
    ///     </description>
    /// </item>
    /// <item>
    ///     <term>matcher</term>
    ///     <description>
    ///     The <see cref="IFileMatcher"/> associated with this processor.
    ///     This property must contains a "classname" key which specifies the fully
    ///     qualified class name of the <see cref="IFileMatcher"/> implementation
    ///     that owns a constructor with nameSpace and prefix arguments.
    ///     If this property doesn't exist, <see cref="AnyFileMatcher"/> is used.
    ///     </description>
    /// </item>
    /// </list>
    /// </para>
    /// </remarks>
    ///
    /// <threadsafety static="true" instance="true"/>
    public abstract class FileProcessorBase : IFileProcessor
    {
        /// <summary>
        /// The property key name for the alias name for the file processor.
        /// </summary>
        ///
        /// <remarks>
        /// The actual property key name is <c>prefix + "alias"</c>.
        /// The property is required and cannot be empty.
        /// </remarks>
        public static readonly string AliasPropertyKey = "alias";

        /// <summary>
        /// The sub-property key name to specify the attached
        /// <see cref="IFileMatcher"/> implementation.
        /// </summary>
        ///
        /// <remarks>
        /// This property combined with <see cref="ClassnamePropertyKey"/>
        /// determines the used <see cref="IFileMatcher"/> implementation.
        /// If not specified, <see cref="AnyFileMatcher"/> is used.
        /// <c>prefix + "matcher"</c> will be passed as the prefix argument
        /// for the <see cref="IFileMatcher"/> implementation, so that
        /// it could have its own configuration under this prefix.
        /// </remarks>
        public static readonly string FileMatcherPropertyKey = "matcher";

        /// <summary>
        /// The sub-property key name to specify the attached
        /// <see cref="IFileMatcher"/> implementation.
        /// </summary>
        ///
        /// <remarks>
        /// This property combined with <see cref="FileMatcherPropertyKey"/>
        /// determines the used <see cref="IFileMatcher"/> implementation.
        /// If not specified, <see cref="AnyFileMatcher"/> is used.
        /// </remarks>
        public static readonly string ClassnamePropertyKey = "classname";

        /// <summary>
        /// This is the alias for the file processor.
        /// It is set in constructor and never be changed.
        /// </summary>
        private string alias;

        /// <summary>
        /// This is the file matcher for the file processor
        /// (which determines if a file can/should be processed).
        /// It is set in constructor and never be changed.
        /// </summary>
        private IFileMatcher matcher;

        /// <summary>
        /// Gets the alias name of the processor.
        /// </summary>
        ///
        /// <remarks>
        /// An alias is the official name of the processor
        /// which is also used to classify the statistic result against.
        /// This should be non-empty.
        /// </remarks>
        ///
        /// <value>
        /// The alias name of the processor.
        /// </value>
        public string Alias
        {
            get
            {
                return this.alias;
            }
        }

        /// <summary>
        /// Gets the file matcher associated with the processor
        /// in order to determine whether a file can/should be processed with
        /// the processor.
        /// </summary>
        ///
        /// <value>
        /// The <see cref="IFileMatcher"/> instance associated with this processor
        /// which determines whether a file can/should be processed with
        /// the processor.
        /// </value>
        public IFileMatcher FileMatcher
        {
            get
            {
                return this.matcher;
            }
        }

        /// <summary>
        /// Gets a list of all the supported metrics by this processor.
        /// </summary>
        ///
        /// <remarks>
        /// This method is mostly for end user. It does not fulfill any
        /// responsibility in the current framework.
        /// </remarks>
        ///
        /// <value>
        /// A list of <see cref="Metric"/> supported by this processor.
        /// </value>
        public abstract IList SupportedMetrics
        {
            get;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="FileProcessorBase"/>
        /// with the specified alias name and <see cref="IFileMatcher"/>.
        /// </summary>
        ///
        /// <param name="alias">
        /// The alias name.
        /// </param>
        /// <param name="matcher">
        /// The <see cref="IFileMatcher"/> to use.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// Any of the argument is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="alias"/> is empty after trimmed.
        /// </exception>
        protected FileProcessorBase(string alias, IFileMatcher matcher)
        {
            // cannot be null
            if (alias == null)
            {
                throw new ArgumentNullException("alias");
            }
            if (matcher == null)
            {
                throw new ArgumentNullException("matcher");
            }
            // cannot be empty
            if (alias.Trim().Length == 0)
            {
                throw new ArgumentException("'alias' cannot be empty", "alias");
            }

            // save to instance members
            this.alias = alias;
            this.matcher = matcher;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="FileProcessorBase"/>
        /// which loads configuration from the specified nameSpace
        /// and prefix of the configuration.
        /// </summary>
        ///
        /// <remarks>
        /// The alias name will be loaded from the property <c>prefix + "alias"</c>,
        /// while the file matcher class will be loaded from the property
        /// <c>prefix + "matcher.classname"</c> passing the nameSpace and
        /// <c>prefix + "matcher."</c> to the file matcher constructor.
        /// If the property <c>prefix + "matcher.classname"</c> is missing,
        /// <see cref="AnyFileMatcher"/> will be used.
        /// </remarks>
        ///
        /// <param name="nameSpace">
        /// The namespace to load configuration values from.
        /// </param>
        /// <param name="prefix">
        /// The prefix of the property name to load configuration values from.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// Any of the argument is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// Any of the argument is empty after trimmed.
        /// </exception>
        /// <exception cref="ConfigurationException">
        /// A configuration error occurs, such as missing alias name,
        /// empty alias name, error instantiating file matcher.
        /// </exception>
        protected FileProcessorBase(string nameSpace, string prefix)
        {
            // cannot be null
            if (nameSpace == null)
            {
                throw new ArgumentNullException("nameSpace");
            }
            if (prefix == null)
            {
                throw new ArgumentNullException("prefix");
            }
            // nameSpace cannot be empty
            if (nameSpace.Trim().Length == 0)
            {
                throw new ArgumentException("'nameSpace' cannot be empty", "nameSpace");
            }

            try
            {
                ConfigManager cm = ConfigManager.GetInstance();

                // get alias (required and cannot be empty)
                string propName = prefix + AliasPropertyKey;
                alias = cm.GetValue(nameSpace, propName);
                if ((alias == null) || (alias.Trim().Length == 0))
                {
                    throw new ConfigurationException(string.Format(
                        "Missing required property: {0} or it is empty", propName));
                }

                // load matcher
                propName = prefix + FileMatcherPropertyKey + ".";
                try
                {
                    // get the file matcher class
                    string matcherClassProp = propName + ClassnamePropertyKey;
                    string matcherClass = cm.GetValue(nameSpace, matcherClassProp);

                    // use AnyFileMatcher if it's missing
                    if (matcherClass == null)
                    {
                        matcher = new AnyFileMatcher();
                    }
                    else
                    {
                        // instantiate as the config
                        // pass the namespace and prefix + FileMatcherPropertyKey + "."
                        matcher = (IFileMatcher) Helper.CreateInstance(cm, nameSpace, null, matcherClassProp,
                            new string[] {nameSpace, propName}, typeof(IFileMatcher));
                    }
                }
                catch (Exception e)
                {
                    throw new ConfigurationException("Error instantiating file matcher", e);
                }
            }
            catch (ConfigurationException)
            {
                // just re-throw
                throw;
            }
            catch (Exception e)
            {
                // encapsulate all others in ConfigurationException
                throw new ConfigurationException(string.Format(
                    "Error initializing from namespace {0}, property prefix {1}",
                    nameSpace, prefix), e);
            }
        }

        /// <summary>
        /// Processes the file and generates the statistic result.
        /// </summary>
        ///
        /// <remarks>
        /// This will actually process the file and gather the file statistic.
        /// The result is then returned via <see cref="StatsResult"/>.
        /// A new result should be generated each time,
        /// whose keys are the metrics returned from <see cref="SupportedMetrics"/>.
        /// </remarks>
        ///
        /// <param name="file">
        /// The file to process.
        /// </param>
        ///
        /// <returns>
        /// A <see cref="StatsResult"/> representing the statistic result for the file.
        /// </returns>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="file"/> is a null reference.
        /// </exception>
        /// <exception cref="IOException">
        /// Any IO operation fails.
        /// </exception>
        /// <exception cref="FileProcessingException">
        /// The processor fails.
        /// </exception>
        public abstract StatsResult Process(FileInfo file);
    }
}
