/*
 * TextLineCounter.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Authors: WishingBone, TCSDEVELOPER
 */

using System;
using System.Collections;
using System.IO;
using TopCoder.File.Statistics;
using TopCoder.File.Statistics.Matcher;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics.Processor.LineCounter
{
    /// <summary>
    /// Counts lines for text files.
    /// </summary>
    ///
    /// <remarks>
    /// It supports a single metric of "Line Count".
    /// It delegates configuration to base class without any addition.
    /// Since version 2.0b this class adds an option to whether ignore a blank line
    /// (line composed of whitespace only).
    /// This option can come from either configuration file or command line.
    /// </remarks>
    ///
    /// <threadsafety static="true" instance="true"/>
    public class TextLineCounter : FileProcessorBase, IArgumentHandler
    {
        /// <summary>
        /// Configuration property key that holds a boolean flag whether to ignore blank line.
        /// </summary>
        ///
        /// <remarks>
        /// This is optional and is evaluated to <b>true</b>
        /// if the value is "true"/"yes"/"on" case sensitively.
        /// Default value is <b>false</b>.
        /// </remarks>
        public static readonly string IgnoreBlankLinePropertyKey = "ignore_blank_line";

        /// <summary>
        /// The name of the metric used for this processor.
        /// </summary>
        public static readonly string Name = "Line Count";

        /// <summary>
        /// The display name of the metric used for this processor.
        /// </summary>
        public static readonly string DisplayName = "line";

        /// <summary>
        /// The description of the metric used for this processor.
        /// </summary>
        private static readonly string description = "Number of lines in a text file.";

        /// <summary>
        /// The help message for the option key.
        /// </summary>
        private static readonly string helpOption =
            "Whether to ignore blank lines (\"yes\"/\"true\"/\"on\" is considered true)";

        /// <summary>
        /// The supported metric list for this processor.
        /// </summary>
        private static readonly Metric[] supportedMetric =
        { new Metric(Name, DisplayName, description) };

        /// <summary>
        /// Flag whether to ignore blank line.
        /// It's set in constructor or HandleArguments(). Default to false.
        /// </summary>
        private bool ignoreBlankLine = false;

        /// <summary>
        /// Gets a list containing a single <see cref="Metric"/> composed of
        /// the <see cref="Name"/>, <see cref="DisplayName"/>, and a description.
        /// </summary>
        ///
        /// <value>
        /// A list containing a single <see cref="Metric"/> composed of
        /// the <see cref="Name"/>, <see cref="DisplayName"/>, and a description.
        /// </value>
        public override IList SupportedMetrics
        {
            get
            {
                // return the supported metric list
                return supportedMetric;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="TextLineCounter"/> instance
        /// with the specified alias name and <see cref="IFileMatcher"/>.
        /// </summary>
        ///
        /// <remarks>
        /// A blank line will not be ignored and considered as 1 line count.
        /// </remarks>
        ///
        /// <param name="alias">
        /// The alias name.
        /// </param>
        /// <param name="matcher">
        /// The <see cref="IFileMatcher"/> to use.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// Any of the argument is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="alias"/> is empty after trimmed.
        /// </exception>
        public TextLineCounter(string alias, IFileMatcher matcher)
            : base(alias, matcher)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="TextLineCounter"/> instance
        /// with the specified alias name, <see cref="IFileMatcher"/>, and ignore blank line flag.
        /// </summary>
        ///
        /// <param name="alias">
        /// The alias name.
        /// </param>
        /// <param name="matcher">
        /// The <see cref="IFileMatcher"/> to use.
        /// </param>
        /// <param name="ignoreBlankLine">
        /// A boolean flag to indicate whether blank line should be ignored.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// Any of the argument is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="alias"/> is empty after trimmed.
        /// </exception>
        public TextLineCounter(string alias, IFileMatcher matcher, bool ignoreBlankLine)
            : base(alias, matcher)
        {
            this.ignoreBlankLine = ignoreBlankLine;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="TextLineCounter"/>
        /// which loads configuration from the specified nameSpace
        /// and prefix of the configuration.
        /// </summary>
        ///
        /// <remarks>
        /// The alias name will be loaded from the property <c>prefix + "alias"</c>,
        /// while the file matcher class will be loaded from the property
        /// <c>prefix + "matcher.classname"</c> passing the nameSpace and
        /// <c>prefix + "matcher."</c> to the file matcher constructor.
        /// If the property <c>prefix + "matcher.classname"</c> is missing,
        /// <see cref="AnyFileMatcher"/> will be used.
        /// Since version 2.0b, there is a support to ignore blank line.
        /// This is specified in the configuration file in the property
        /// <c>prefix + "ignore_blank_line"</c>.
        /// The value "true"/"yes"/"on" case insensitively is considered true.
        /// It is optional and default to false.
        /// </remarks>
        ///
        /// <param name="nameSpace">
        /// The namespace to load configuration values from.
        /// </param>
        /// <param name="prefix">
        /// The prefix of the property name to load configuration values from.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// Any of the argument is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// Any of the argument is empty after trimmed.
        /// </exception>
        /// <exception cref="ConfigurationException">
        /// A configuration error occurs, such as missing alias name,
        /// empty alias name, error instantiating file matcher.
        /// </exception>
        public TextLineCounter(string nameSpace, string prefix)
            : base(nameSpace, prefix)
        {
            try
            {
                ConfigManager cm = ConfigManager.GetInstance();

                // get ignore_blank_line
                string propName = prefix + IgnoreBlankLinePropertyKey;
                string ignore = cm.GetValue(nameSpace, propName);
                if (ignore != null)
                {
                    // true if "yes"/"on"/"true" case-insensitively
                    ignoreBlankLine = Helper.BoolValue(ignore);
                }
            }
            catch (Exception e)
            {
                // encapsulate all in ConfigurationException
                throw new ConfigurationException(string.Format(
                    "Error initializing from namespace {0}, property prefix {1}",
                    nameSpace, prefix), e);
            }
        }

        /// <summary>
        /// Count the lines in the text file.
        /// </summary>
        ///
        /// <remarks>
        /// This will assume that the specified file is a text file.
        /// It would open the file as a text stream and count the lines until EOF is met.
        /// Since version 2.0b, there is an option to ignore blank lines.
        /// </remarks>
        ///
        /// <param name="file">
        /// The file to process.
        /// </param>
        ///
        /// <returns>
        /// A <see cref="StatsResult"/> representing the number of lines in the file.
        /// </returns>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="file"/> is a null reference.
        /// </exception>
        /// <exception cref="IOException">
        /// Any IO operation fails.
        /// </exception>
        /// <exception cref="FileProcessingException">
        /// The processor fails.
        /// </exception>
        public override StatsResult Process(FileInfo file)
        {
            // cannot be null
            if (file == null)
            {
                throw new ArgumentNullException("file");
            }

            try
            {
                // open as text and count the line
                long count = 0;
                using (StreamReader stream = file.OpenText())
                {
                    string line = stream.ReadLine();
                    while (line != null)
                    {
                        if (!ignoreBlankLine)
                        {
                            count++;
                        }
                        else
                        {
                            // ignore blank line
                            if (line.Trim().Length > 0)
                            {
                                count++;
                            }
                        }
                        line = stream.ReadLine();
                    }
                }

                // return the StatsResult
                StatsResult result = new StatsResult();
                result[(Metric) SupportedMetrics[0]] = count;
                return result;
            }
            catch (IOException)
            {
                // just re-throw
                throw;
            }
            catch (Exception e)
            {
                // encapsulate all others in IOException
                throw new IOException(string.Format(
                    "Error processing file {0}", file.FullName), e);
            }
        }

        /// <summary>
        /// Proposes a single <see cref="ArgumentOption"/> with "ignore_blank_line"
        /// as switch name and non-required option.
        /// </summary>
        ///
        /// <returns>
        /// A <see cref="IList"/> containing a single <see cref="ArgumentOption"/>
        /// with "ignore_blank_line" as switch name and non-required option.
        /// </returns>
        public IList ProposeOptions()
        {
            IList options = new ArrayList();
            options.Add(new ArgumentOption(IgnoreBlankLinePropertyKey, helpOption, false));
            return options;
        }

        /// <summary>
        /// Receives command line argument from the command line interface.
        /// </summary>
        ///
        /// <remarks>
        /// It does not care about the actual value of the switch (which is generally not provided).
        /// It only cares about whether the key option exists (on) or not (off).
        /// </remarks>
        ///
        /// <param name="arguments">
        /// The arguments from command line.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="arguments"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// If the "ignore_blank_line" value is not a String.
        /// </exception>
        public void HandleArguments(IDictionary arguments)
        {
            if (arguments == null)
            {
                throw new ArgumentNullException("arguments");
            }
            object ignore = arguments[IgnoreBlankLinePropertyKey];
            if ((ignore != null) && (!(ignore is string)))
            {
                throw new ArgumentException("'ignore_blank_line' value is not of type string", "arguments");
            }

            // turn on option if the key exists
            if (ignore != null)
            {
                ignoreBlankLine = true;
            }
        }
    }
}
