/*
 * AbstractStatsReporting.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Authors: WishingBone, TCSDEVELOPER
 */

using System;
using System.Collections;
using System.IO;

namespace TopCoder.File.Statistics.Reporting
{
    /// <summary>
    /// Provides a <see cref="IStatsReporting"/> implementation base which
    /// aggregates results from each processor and classifies them based on
    /// single file, directory, or alias and also provides an overall stats.
    /// </summary>
    ///
    /// <remarks>
    /// <para>
    /// The actual rendering logic is defered to concrete implementations
    /// (<see cref="BasicStatsReporting"/> and <see cref="XmlStatsReporting"/>).
    /// </para>
    /// <para>
    /// It is possible to initialize this class using reflection
    /// with the constructor that takes the nameSpace and prefix arguments.
    /// This constructor is provided for extension purpose only.
    /// No work is done by this constructor.
    /// </para>
    /// </remarks>
    ///
    /// <threadsafety static="false" instance="false"/>
    public abstract class AbstractStatsReporting : IStatsReporting
    {
        /// <summary>
        /// The stats mapped to each file.
        /// The file path is relative to base dir or absolute if no base dir.
        /// Its reference never changes.
        /// </summary>
        private StatsCollection fileStats = new StatsCollection();

        /// <summary>
        /// The stats mapped to each directory.
        /// The dir path is relative to base dir or absolute if no base dir.
        /// Its reference never changes.
        /// </summary>
        private StatsCollection directoryStats = new StatsCollection();

        /// <summary>
        /// The stats mapped to each processor alias name.
        /// </summary>
        private StatsCollection aliasStats = new StatsCollection();

        /// <summary>
        /// The overall stats. It is created new when reporting resets.
        /// </summary>
        private StatsResult overallStats = new StatsResult();

        /// <summary>
        /// The base dir for the reporting.
        /// This variable is nullable (meaning all paths are absolute).
        /// Read via property getter, set via Reset().
        /// 2 possible DirectoryInfo:
        /// - new DirectoryInfo("test_files\"); (end with a backslash)
        /// - new DirectoryInfo("test_files");  (no backslash)
        /// ExtractRelative will deal with these.
        /// </summary>
        private DirectoryInfo baseDirectory = null;

        /// <summary>
        /// Gets the file statistic.
        /// </summary>
        ///
        /// <value>
        /// A <see cref="StatsCollection"/> where the key is the file name
        /// and the value is its <see cref="StatsResult"/>.
        /// </value>
        public StatsCollection FileStats
        {
            get
            {
                return fileStats;
            }
        }

        /// <summary>
        /// Gets the directory statistic.
        /// </summary>
        ///
        /// <value>
        /// A <see cref="StatsCollection"/> where the key is the directory name
        /// and the value is its <see cref="StatsResult"/>.
        /// </value>
        public StatsCollection DirectoryStats
        {
            get
            {
                return directoryStats;
            }
        }

        /// <summary>
        /// Gets the (processor) alias statistic.
        /// </summary>
        ///
        /// <value>
        /// A <see cref="StatsCollection"/> where the key is the alias name
        /// and the value is its <see cref="StatsResult"/>.
        /// </value>
        public StatsCollection AliasStats
        {
            get
            {
                return aliasStats;
            }
        }

        /// <summary>
        /// Gets the overal statistic.
        /// </summary>
        ///
        /// <value>
        /// A <see cref="StatsResult"/> representing the overal statistic.
        /// </value>
        public StatsResult OverallStats
        {
            get
            {
                // return a clone
                return new StatsResult(overallStats);
            }
        }

        /// <summary>
        /// Gets the base directory.
        /// </summary>
        ///
        /// <value>
        /// A <see cref="DirectoryInfo"/> representing the base directory
        /// or <b>null</b> if no base directory is established.
        /// </value>
        public DirectoryInfo BaseDirectory
        {
            get
            {
                return baseDirectory;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="AbstractStatsReporting"/> class.
        /// </summary>
        protected AbstractStatsReporting()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="AbstractStatsReporting"/>
        /// which loads configuration from the specified nameSpace
        /// and prefix of the configuration.
        /// </summary>
        ///
        /// <remarks>
        /// This constructor is provided for extension purpose only.
        /// No configuration is processed by this constructor.
        /// </remarks>
        ///
        /// <param name="nameSpace">
        /// The namespace to load configuration values from.
        /// </param>
        /// <param name="prefix">
        /// The prefix of the property name to load configuration values from.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// Any of the argument is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// Any of the argument is empty after trimmed.
        /// </exception>
        /// <exception cref="ConfigurationException">
        /// A configuration error occurs.
        /// </exception>
        protected AbstractStatsReporting(string nameSpace, string prefix)
        {
            // cannot be null
            if (nameSpace == null)
            {
                throw new ArgumentNullException("nameSpace");
            }
            if (prefix == null)
            {
                throw new ArgumentNullException("prefix");
            }
            // nameSpace cannot be empty
            if (nameSpace.Trim().Length == 0)
            {
                throw new ArgumentException("'nameSpace' cannot be empty", "nameSpace");
            }
        }

        /// <summary>
        /// Aggregates the result into the reporting.
        /// </summary>
        ///
        /// <remarks>
        /// This will aggregate the specified <paramref name="result"/>
        /// based on its file name, all the paths its in, and processor's alias.
        /// The overal statistic will also be aggregated.
        /// </remarks>
        ///
        /// <param name="processor">
        /// The processor that generated the result.
        /// </param>
        /// <param name="file">
        /// The file that is processed.
        /// </param>
        /// <param name="result">
        /// The processing result
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// Any of the argument is a null reference.
        /// </exception>
        /// <exception cref="StatsReportingException">
        /// <paramref name="file"/> is not in base dir.
        /// </exception>
        public void Aggregate(IFileProcessor processor, FileInfo file, StatsResult result)
        {
            // cannot be null
            if (processor == null)
            {
                throw new ArgumentNullException("processor");
            }
            if (file == null)
            {
                throw new ArgumentNullException("file");
            }
            if (result == null)
            {
                throw new ArgumentNullException("result");
            }

            // aggregate file name
            // use the absolute file name
            // because it's possible for a filename to exist in two different directories
            string fileName = ExtractRelative(file.FullName);
            if (fileName == null)
            {
                throw new StatsReportingException(string.Format(
                    "File {0} is not under base directory {1}", file.FullName,
                    ((BaseDirectory == null) ? "\\" : BaseDirectory.FullName)));
            }
            fileStats.Add(fileName, result);

            // aggregate directory up to, not including, its base directory
            DirectoryInfo dir = file.Directory;
            string dirName = ExtractRelative(dir.FullName);
            while ((dirName != null) && (dirName.Length > 0))
            {
                directoryStats.Add(dirName, result);

                // get the parent
                dir = dir.Parent;
                if (dir == null)
                {
                    dirName = null;
                }
                else
                {
                    dirName = ExtractRelative(dir.FullName);
                }
            }

            // aggregate alias stat
            aliasStats.Add(processor.Alias, result);

            // aggregate overal
            overallStats.Aggregate(result);
        }

        /// <summary>
        /// Clears all the aggregated result and sets the base dir (which is nullable).
        /// </summary>
        ///
        /// <param name="baseDirectory">
        /// The base directory to reset and for aggregating report.
        /// <b>null</b> means absolute paths are desirable.
        /// </param>
        ///
        /// <exception cref="ArgumentException">
        /// <paramref name="baseDirectory"/> is not a valid directory.
        /// </exception>
        public void Reset(DirectoryInfo baseDirectory)
        {
            // check directory exists and valid
            if ((baseDirectory != null) && (!(baseDirectory.Exists)))
            {
                throw new ArgumentException(string.Format(
                    "Directory {0} doesn't exist", baseDirectory.FullName));
            }

            // clears all result
            fileStats.Clear();
            directoryStats.Clear();
            aliasStats.Clear();
            overallStats = new StatsResult();

            // save to instance member
            this.baseDirectory = baseDirectory;
        }

        /// <summary>
        /// Generates the report with all the aggregated results.
        /// </summary>
        ///
        /// <returns>
        /// The generated report.
        /// </returns>
        ///
        /// <exception cref="StatsReportingException">
        /// An error occurs.
        /// </exception>
        public abstract string GenerateReport();

        /// <summary>
        /// Makes a relative path of the specified full path from the BaseDirectory.
        /// </summary>
        ///
        /// <param name="fullPath">
        /// The fullpath of a file or a directory.
        /// </param>
        ///
        /// <returns>
        /// A relative path (from the BaseDirectory) of the fullPath
        /// or empty string if fullPath is equal to base directory
        /// or <b>null</b> if fullPath is not under base directory or is null.
        /// </returns>
        private string ExtractRelative(string fullPath)
        {
            // if base is null, always returns the fullPath
            if (BaseDirectory == null)
            {
                return fullPath;
            }

            // if fullpath is null, return null
            if (fullPath == null)
            {
                return null;
            }

            // get the BaseDirectory fullname to compare
            string baseDir = BaseDirectory.FullName;
            // append \ so that it won't match a file with similar name to the dir
            if (!baseDir.EndsWith(@"\"))
            {
                baseDir += @"\";
            }

            if (baseDir == fullPath)
            {
                // BaseDirectory is equal to checkDir
                return "";
            }
            else if (fullPath.StartsWith(baseDir))
            {
                // fullPath is under BaseDirectory
                return fullPath.Substring(baseDir.Length);
            }

            // fullPath is not under BaseDirectory
            return null;
        }
    }
}