///
/// $(#)MimeMatcherTests.java		Sep 6, 2005
/// 
/// Copyright (c) 2005, TopCoder Inc. All rights reserved.
/// </summary>

using System;
using System.IO;
using System.Collections;
using NUnit.Framework;

using TopCoder.File.Statistics.Matcher;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics.FailureTests.Matcher
{
	/// <summary>
    /// This test fixture contains the failure tests for <see cref="BinaryFileMatcher"/> class.
    ///
    /// <author>MPhk</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    [TestFixture]
	public class MimeMatcherTests {
	
	    /// <summary>
	    /// The namespace for this test.
	    /// </summary>
	    private const string NAMESPACE = "TopCoder.File.Statistics.Matcher.FailureTests";
	
	    /// <summary>
	    /// The config file for this test.
	    /// </summary>
	    private const string CONFIG_FILE = "../../test_files/failuretests/matcher/config.xml";
	
	    /// <summary>
	    /// The namespace for Magic Numbers component.
	    /// </summary>
	    private const string MAGIC_NUMBERS_CONFIG = "../../test_files/failuretests/MagicNumbers.xml";
	
	    /// <summary>
	    /// The the base path of the test files.
	    /// </summary>
	    private const string BASE_PATH = "../../test_files/failuretests/matcher/";
	
	    /// <summary>
	    /// A MimeMatcher instance to test.
	    /// </summary>
	    private MimeMatcher matcher = null;
	
	    /// <summary>
	    /// Load the configuration before testing.
	    /// </summary>
	    [SetUp]
	    protected void SetUp() {
	        // clear the namespace first
	        ConfigManager configManager = ConfigManager.GetInstance();
	        configManager.RemoveFile(CONFIG_FILE);
	        // load the configuration
	        configManager.LoadFile(CONFIG_FILE);
	        configManager.RemoveFile(MAGIC_NUMBERS_CONFIG);
	        configManager.LoadFile(MAGIC_NUMBERS_CONFIG);
	        
	        matcher = new MimeMatcher("image/gif");
	    }
	
	    /// <summary>
	    /// Remove the configuration after testing.
	    /// </summary>
	    [TearDown]
	    protected void TearDown() {
	        ConfigManager configManager = ConfigManager.GetInstance();
	        configManager.RemoveFile(CONFIG_FILE);
	        configManager.RemoveFile(MAGIC_NUMBERS_CONFIG);
	    }
	
	    /// <summary>
	    /// Checks whether the constructor MimeMatcher(Pattern) properly throws
	    /// ArgumentNullException for null argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentNullException))]
	    public void TestMimeMatcher1() {
            new MimeMatcher((String) null);
	    }
	    
	    /// <summary>
	    /// Checks whether the constructor MimeMatcher(Pattern) properly throws
	    /// ArgumentException for empty String argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentException))]
	    public void TestMimeMatcher2() {
            new MimeMatcher("");
	    }
	
	    /// <summary>
	    /// Checks whether the constructor MimeMatcher(string, string) properly throws
	    /// ArgumentNullException for null argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentNullException))]
	    public void TestMimeMatcher3() {
            new MimeMatcher(null, "prefix");
	    }
	    
	    /// <summary>
	    /// Checks whether the constructor MimeMatcher(string, string) properly throws
	    /// ArgumentException for empty argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentException))]
	    public void TestMimeMatcher4() {
            new MimeMatcher("", "prefix");
	    }
	    
	    /// <summary>
	    /// Checks whether the constructor MimeMatcher(string, string) properly throws
	    /// ArgumentNullException for null argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentNullException))]
	    public void TestMimeMatcher5() {
            new MimeMatcher("namespace", null);
	    }
	
	    /// <summary>
	    /// Checks whether the constructor MimeMatcher(Property) properly throws
	    /// Configuration for invalid property - no values in "mimes" sub-property.
	    /// </summary>
	    [Test, ExpectedException(typeof(ConfigurationException))]
	    public void TestMimeMatcher6() {
            new MimeMatcher(NAMESPACE, "mimes_invalid1");
	    }
	
	    /// <summary>
	    /// Checks whether the constructor MimeMatcher(Property) properly throws
	    /// Configuration for invalid property - no "mimes" sub-property.
	    /// </summary>
	    [Test, ExpectedException(typeof(ConfigurationException))]
	    public void TestMimeMatcher7() {
            new MimeMatcher(NAMESPACE, "mimes_invalid2");
	    }
	
	    /// <summary>
	    /// Checks whether the constructor MimeMatcher(Property) properly throws
	    /// Configuration for invalid property - empty values.
	    /// </summary>
	    [Test, ExpectedException(typeof(ConfigurationException))]
	    public void TestMimeMatcher8() {
            new MimeMatcher(NAMESPACE, "mimes_invalid3");
	    }
	    
	    /// <summary>
	    /// Checks whether the constructor MimeMatcher(List) properly throws
	    /// ArgumentNullException for null argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentNullException))]
	    public void TestMimeMatcher9() {
            new MimeMatcher((IList) null);
	    }
	
	    /// <summary>
	    /// Checks whether the constructor MimeMatcher(List) properly throws
	    /// ArgumentException for empty List argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentException))]
	    public void TestMimeMatcher10() {
            new MimeMatcher(new ArrayList());
	    }
	
	    /// <summary>
	    /// Checks whether the constructor MimeMatcher(List) properly throws
	    /// ArgumentException for List argument containing nulls.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentException))]
	    public void TestMimeMatcher11() {
	        IList list = new ArrayList();
	        list.Add("1");
	        list.Add(null);
	
            new MimeMatcher(list);
	    }
	
	    /// <summary>
	    /// Checks whether the constructor MimeMatcher(List) properly throws
	    /// ArgumentException for List argument containing empty Strings.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentException))]
	    public void TestMimeMatcher12() {
	        IList list = new ArrayList();
	        list.Add("1");
	        list.Add("");
	
            new MimeMatcher(list);
	    }
	
	    /// <summary>
	    /// Checks whether the constructor MimeMatcher(List) properly throws
	    /// ArgumentException for List argument containing non String elements.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentException))]
	    public void TestMimeMatcher13() {
	        IList list = new ArrayList();
	        list.Add("1");
	        list.Add(new Object());
	
            new MimeMatcher(list);
	    }
	
	    /// <summary>
	    /// Tests the match method with null argument. ArgumentNullException should be thrown.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentNullException))]
	    public void TestMatch1() {
            matcher.Match((FileInfo) null);
	    }
	
	    /// <summary>
	    /// Tests the match method for inexistent file. IOException should be thrown.
	    /// </summary>
	    [Test, ExpectedException(typeof(IOException))]
	    public void TestMatch2() {
                try {
                    matcher.Match(new FileInfo(BASE_PATH + "/inexistent_file"));
                } catch (IOException)
                {
                    throw new IOException();
                }
	    }
	
	    /// <summary>
	    /// Tests the match method for a folder. IOException should be thrown.
	    /// </summary>
	    [Test, ExpectedException(typeof(IOException))]
	    public void TestMatch3() {
                try {
                    matcher.Match(new FileInfo(BASE_PATH));
                } catch (IOException)
                {
                    throw new IOException();
                }
	    }
    }
}
