/*
 * TestFileNameMatcher.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using System.IO;
using NUnit.Framework;
using TopCoder.File.Statistics.Tests;

namespace TopCoder.File.Statistics.Matcher.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="FileNameMatcher"/> abstract class.
    /// </summary>
    [TestFixture]
    public class TestFileNameMatcher : TestHelper
    {
        /// <summary>
        /// A concrete class inherited from <see cref="FileNameMatcher"/>
        /// which is used by <see cref="TestFileNameMatcher"/>
        /// to test <see cref="FileNameMatcher"/>.
        /// </summary>
        ///
        /// <remarks>
        /// This class only implements the abstract member Match
        /// by saving the passed filename so that it can be asserted
        /// using the <see cref="FileName"/> property.
        /// </remarks>
        public class FileNameMatcherClass : FileNameMatcher
        {
            /// <summary>
            /// Initializes a new instance of the <see cref="FileNameMatcherClass"/> instance.
            /// </summary>
            public FileNameMatcherClass()
            {
            }

            /// <summary>
            /// The filename passed to the template method.
            /// </summary>
            private string filename;

            /// <summary>
            /// Gets the filename passed to the template method.
            /// </summary>
            ///
            /// <value>
            /// The filename passed to the template method.
            /// </value>
            public string FileName
            {
                get
                {
                    return filename;
                }
            }

            /// <summary>
            /// Saves the filename so that it can be asserted.
            /// </summary>
            ///
            /// <param name="filename">
            /// The filename to save.
            /// </param>
            ///
            /// <returns>
            /// Always <b>true</b>.
            /// </returns>
            protected override bool Match(string filename)
            {
                // save the filename
                this.filename = filename;
                return true;
            }
        }

        /// <summary>
        /// The <see cref="FileNameMatcherClass"/> instance to test.
        /// Initialized in SetUp().
        /// </summary>
        private FileNameMatcherClass matcher;

        /// <summary>
        /// Initializes the <see cref="FileNameMatcherClass"/> instance to test.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            matcher = new FileNameMatcherClass();
        }

        /// <summary>
        /// Tests constructor() doesn't throw any exception.
        /// </summary>
        [Test]
        public void TestConstructor1()
        {
            new FileNameMatcherClass();
        }

        /// <summary>
        /// Tests Match(FileInfo) extract the filename correctly.
        /// </summary>
        [Test]
        public void TestMatch()
        {
            // a valid file
            matcher.Match(new FileInfo(SampleText));

            // assert the extracted file name
            Assert.AreEqual("test.txt", matcher.FileName,
                "The filename is incorrectly extracted");
        }

        /// <summary>
        /// Tests Match() given non-existing file.
        /// </summary>
        [Test]
        public void TestMatchNotExist()
        {
            // the file doesn't exist
            matcher.Match(new FileInfo(@"..\..\test_files\notexists.txt"));

            // assert the extracted file name
            Assert.AreEqual("notexists.txt", matcher.FileName,
                "The filename is incorrectly extracted");
        }

        /// <summary>
        /// Tests Match() given an existing directory.
        /// </summary>
        [Test]
        public void TestMatchDirectory()
        {
            // an existing directory
            matcher.Match(new FileInfo(@"..\..\test_files"));

            // assert that the directory name is extracted
            Assert.AreEqual("test_files", matcher.FileName,
                "The directory name is extracted");
        }

        /// <summary>
        /// Tests Match() given a non existing directory.
        /// </summary>
        [Test]
        public void TestMatchDirectoryNotExist()
        {
            // a non-existing directory
            matcher.Match(new FileInfo(@"..\..\notexists"));

            // assert that the directory name is extracted
            Assert.AreEqual("notexists", matcher.FileName,
                "The directory name is extracted");
        }

        /// <summary>
        /// Tests Match() throws ArgumentNullException
        /// given null FileInfo.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestMatchNull()
        {
            matcher.Match(null);
        }
    }
}
