/*
 * TestFileTypeMatcher.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using System.IO;
using NUnit.Framework;
using TopCoder.File.Statistics.Tests;
using TopCoder.Unix.MagicNumbers;

namespace TopCoder.File.Statistics.Matcher.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="FileTypeMatcher"/> abstract class.
    /// </summary>
    [TestFixture]
    public class TestFileTypeMatcher : TestHelper
    {
        /// <summary>
        /// A concrete class inherited from <see cref="FileTypeMatcher"/>
        /// which is used by <see cref="TestFileTypeMatcher"/>
        /// to test <see cref="FileTypeMatcher"/>.
        /// </summary>
        ///
        /// <remarks>
        /// This class only implements the abstract member Match
        /// by saving the passed FileType so that it can be asserted
        /// using the <see cref="Type"/> property.
        /// </remarks>
        public class FileTypeMatcherClass : FileTypeMatcher
        {
            /// <summary>
            /// Initializes a new instance of the <see cref="FileTypeMatcherClass"/> instance.
            /// </summary>
            public FileTypeMatcherClass()
            {
            }

            /// <summary>
            /// The FileType passed to the template method.
            /// </summary>
            private FileType type;

            /// <summary>
            /// Gets the FileType passed to the template method.
            /// </summary>
            ///
            /// <value>
            /// The FileType passed to the template method.
            /// </value>
            public FileType Type
            {
                get
                {
                    return type;
                }
            }

            /// <summary>
            /// Saves the flag so that it can be asserted.
            /// </summary>
            ///
            /// <param name="type">
            /// The file type to save.
            /// </param>
            ///
            /// <returns>
            /// Always <b>true</b>.
            /// </returns>
            protected override bool Match(FileType type)
            {
                // save the flag
                this.type = type;
                return true;
            }
        }

        /// <summary>
        /// The <see cref="FileTypeMatcherClass"/> instance to test.
        /// Initialized in SetUp().
        /// </summary>
        private FileTypeMatcherClass matcher;

        /// <summary>
        /// Initializes the <see cref="FileTypeMatcherClass"/> instance to test.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            // load magic number configuration
            base.LoadConfigFile(MagicNumberConfig);

            // initialize FileTypeMatcherClass
            matcher = new FileTypeMatcherClass();
        }

        /// <summary>
        /// Tests constructor() doesn't throw any exception.
        /// </summary>
        [Test]
        public void TestConstructor1()
        {
            new FileTypeMatcherClass();
        }

        /// <summary>
        /// Tests Match(FileInfo) with a java class file.
        /// </summary>
        [Test]
        public void TestMatchBinary()
        {
            // a java class file
            matcher.Match(new FileInfo(@"..\..\test_files\Matcher_FileType_MagicNumber.class"));

            // assert type is "Java class file"
            Assert.AreEqual("Java class file", matcher.Type.Name,
                "The file type should be detected as a java class file");
        }

        /// <summary>
        /// Tests Match(FileInfo) with an xml text file.
        /// </summary>
        [Test]
        public void TestMatchText()
        {
            // an xml text file
            matcher.Match(new FileInfo(@"..\..\test_files\MagicNumber.xml"));

            // assert type is "XML file"
            Assert.AreEqual("XML file", matcher.Type.Name,
                "The file type should be detected as an xml file");
        }

        /// <summary>
        /// Tests Match() throws IOException
        /// given a non-existing file.
        /// </summary>
        [Test]
        public void TestMatchNotExist()
        {
            try
            {
                // the file doesn't exist
                // this may throw FileNotFoundException, which is a subclass of IOException
                matcher.Match(new FileInfo(@"..\..\test_files\notexists.txt"));
                Assert.Fail("IOException expected");
            }
            catch (IOException)
            {
                // ok
            }
        }

        /// <summary>
        /// Tests Match() throws IOException
        /// given an existing directory.
        /// </summary>
        [Test]
        [ExpectedException(typeof(IOException))]
        public void TestMatchDirectory()
        {
            // an existing directory
            matcher.Match(new FileInfo(@"..\..\test_files"));
        }

        /// <summary>
        /// Tests Match() throws FileMatchingException
        /// if Magic Numbers configuration is not found.
        /// </summary>
        [Test]
        [ExpectedException(typeof(FileMatchingException))]
        public void TestMatchMagicNumberError()
        {
            // remove all configuration
            manager.Clear();

            // match a file
            matcher.Match(new FileInfo(SampleText));
        }

        /// <summary>
        /// Tests Match() throws ArgumentNullException
        /// given null FileInfo.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestMatchNull()
        {
            matcher.Match(null);
        }
    }
}
