/*
 * TestMimeMatcher.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using System.Collections;
using System.IO;
using NUnit.Framework;
using TopCoder.File.Statistics.Tests;

namespace TopCoder.File.Statistics.Matcher.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="MimeMatcher"/> class.
    /// </summary>
    [TestFixture]
    public class TestMimeMatcher : TestHelper
    {
        /// <summary>
        /// The <see cref="MimeMatcher"/> instance to test.
        /// Initialized in each unit test case according to its need.
        /// </summary>
        private IFileMatcher matcher;

        /// <summary>
        /// A list of MIMEs to match.
        /// Initialized in SetUp() and used to test constructor(IList).
        /// </summary>
        private IList mimes;

        /// <summary>
        /// Initializes the <see cref="MimeMatcher"/> instance to test
        /// using the test_files/Matcher_Mime.xml.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            // load Magic Number configuration
            // load test_files/Matcher_Mime.xml which is
            // used for constructor with nameSpace and prefix argument
            base.LoadConfigFile(MagicNumberConfig, @"..\..\test_files\Matcher_Mime.xml");

            // initialize type names list (with "text/xml" and "text/tcs")
            mimes = new ArrayList();
            mimes.Add("text/xml");
            mimes.Add("text/tcs");
        }

        /// <summary>
        /// Tears down configuration manager.
        /// </summary>
        [TearDown]
        public void TearDown()
        {
            manager.Clear();
        }

        /// <summary>
        /// Tests the static property key name.
        /// </summary>
        [Test]
        public void TestStaticPropertyKey()
        {
            Assert.AreEqual("mimes", MimeMatcher.MimesPropertyKey,
                "The property key name is wrong");
        }

        /// <summary>
        /// Tests constructor(string) is initialized with a single mime name
        /// and Match() only returns true for the specified mime type.
        /// </summary>
        [Test]
        public void TestConstructor1()
        {
            // XML file
            matcher = new MimeMatcher("text/xml");

            // txt, java class file should not match
            Assert.IsFalse(matcher.Match(new FileInfo(SampleText)),
                "txt should not match");
            Assert.IsFalse(matcher.Match(new FileInfo(@"..\..\test_files\Matcher_FileType_MagicNumber.class")),
                "java class file should not match");

            // xml file should match
            Assert.IsTrue(matcher.Match(new FileInfo(@"..\..\test_files\FileStatistics.xml")),
                "XML file should match");
        }

        /// <summary>
        /// Tests constructor(string) is initialized with a mime name that is
        /// wrong in case and doesn't match that defined in Magic Number.
        /// </summary>
        [Test]
        public void TestConstructor1Case()
        {
            // xml file (wrong case)
            matcher = new MimeMatcher("text/XML");

            // txt, java class file should not match
            Assert.IsFalse(matcher.Match(new FileInfo(SampleText)),
                "txt should not match");
            Assert.IsFalse(matcher.Match(new FileInfo(@"..\..\test_files\Matcher_FileType_MagicNumber.class")),
                "java class file should not match");

            // xml file should not match either
            Assert.IsFalse(matcher.Match(new FileInfo(@"..\..\test_files\FileStatistics.xml")),
                "XML file should not match because the case is wrong");
        }

        /// <summary>
        /// Tests constructor(string) throws ArgumentNullException
        /// given null argument.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructor1Null()
        {
            new MimeMatcher((string) null);
        }

        /// <summary>
        /// Tests constructor(string) throws ArgumentException
        /// given empty argument.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructor1Empty()
        {
            new MimeMatcher(" ");
        }

        /// <summary>
        /// Tests constructor(IList) is initialized with two MIMEs
        /// and Match() only returns true for the specified type.
        /// </summary>
        [Test]
        public void TestConstructor2()
        {
            // text/xml and text/tcs
            matcher = new MimeMatcher(mimes);

            // txt, java class file should not match
            Assert.IsFalse(matcher.Match(new FileInfo(SampleText)),
                "txt should not match");
            Assert.IsFalse(matcher.Match(new FileInfo(@"..\..\test_files\Matcher_FileType_MagicNumber.class")),
                "Java class file should not match");

            // xml file, tcs file should match
            Assert.IsTrue(matcher.Match(new FileInfo(@"..\..\test_files\FileStatistics.xml")),
                "XML file should match");
            Assert.IsTrue(matcher.Match(new FileInfo(@"..\..\test_files\Matcher_FileType_Text.tcs")),
                "TCS file should match");
        }

        /// <summary>
        /// Tests constructor(IList) is initialized with a MIME that is wrong in case
        /// and doesn't match that defined in Magic Number.
        /// </summary>
        [Test]
        public void TestConstructor2Case()
        {
            // Magic number defines "text/xml"
            mimes.Clear();
            mimes.Add("text/XML");
            matcher = new MimeMatcher(mimes);

            // all should not match
            Assert.IsFalse(matcher.Match(new FileInfo(SampleText)),
                "txt should not match");
            Assert.IsFalse(matcher.Match(new FileInfo(@"..\..\test_files\Matcher_FileType_MagicNumber.class")),
                "Java class file should not match");
            Assert.IsFalse(matcher.Match(new FileInfo(@"..\..\test_files\FileStatistics.xml")),
                "XML file should not match");
            Assert.IsFalse(matcher.Match(new FileInfo(@"..\..\test_files\Matcher_FileType_Text.tcs")),
                "TCS file should not match");
        }

        /// <summary>
        /// Tests constructor(IList) throws ArgumentNullException
        /// given null argument.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructor2Null()
        {
            new MimeMatcher((IList) null);
        }

        /// <summary>
        /// Tests constructor(IList) throws ArgumentException
        /// given empty list.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructor2List()
        {
            new MimeMatcher(new ArrayList());
        }

        /// <summary>
        /// Tests constructor(IList) throws ArgumentException
        /// given list containing null element.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructor2NullElement()
        {
            mimes.Add(null);
            new MimeMatcher(mimes);
        }

        /// <summary>
        /// Tests constructor(IList) throws ArgumentException
        /// given list containing non-string element.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructor2NonStringElement()
        {
            mimes.Add(123);
            new MimeMatcher(mimes);
        }

        /// <summary>
        /// Tests constructor(IList) throws ArgumentException
        /// given list containing empty string element.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructor2EmptyStringElement()
        {
            mimes.Add(" ");
            new MimeMatcher(mimes);
        }

        /// <summary>
        /// Tests constructor(IList) throws ArgumentException
        /// given list containing duplicate element.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructor2DuplicateElement()
        {
            mimes.Add(mimes[0]);
            new MimeMatcher(mimes);
        }

        /// <summary>
        /// Tests constructor(string, string) is initialized with two MIMEs
        /// and Match() only returns true for the specified types.
        /// </summary>
        [Test]
        public void TestConstructor3()
        {
            // initialize MimeMatcher ("text/xml" and "text/tcs")
            matcher = new MimeMatcher(ValidNamespace, "matcher.");

            // txt, java class file should not match
            Assert.IsFalse(matcher.Match(new FileInfo(SampleText)),
                "txt should not match");
            Assert.IsFalse(matcher.Match(new FileInfo(@"..\..\test_files\Matcher_FileType_MagicNumber.class")),
                "Java class file should not match");

            // xml file, tcs file should match
            Assert.IsTrue(matcher.Match(new FileInfo(@"..\..\test_files\FileStatistics.xml")),
                "XML file should match");
            Assert.IsTrue(matcher.Match(new FileInfo(@"..\..\test_files\Matcher_FileType_Text.tcs")),
                "TCS file should match");
        }

        /// <summary>
        /// Tests constructor(string, string) is initialized with a MIME that is wrong in case
        /// and doesn't match that defined in Magic Number.
        /// </summary>
        [Test]
        public void TestConstructor3Case()
        {
            // initialize MimeMatcher ("text/XML")
            matcher = new MimeMatcher(ValidNamespace, "matcher_incorrect.");

            // all should not match
            Assert.IsFalse(matcher.Match(new FileInfo(SampleText)),
                "txt should not match");
            Assert.IsFalse(matcher.Match(new FileInfo(@"..\..\test_files\Matcher_FileType_MagicNumber.class")),
                "Java class file should not match");
            Assert.IsFalse(matcher.Match(new FileInfo(@"..\..\test_files\FileStatistics.xml")),
                "XML file should not match");
            Assert.IsFalse(matcher.Match(new FileInfo(@"..\..\test_files\Matcher_FileType_Text.tcs")),
                "TCS file should not match");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ArgumentNullException
        /// given null nameSpace.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructor3NullNamespace()
        {
            new MimeMatcher(null, "matcher.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ArgumentNullException
        /// given null prefix.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructor3NullPrefix()
        {
            new MimeMatcher(ValidNamespace, null);
        }

        /// <summary>
        /// Tests constructor(string, string) throws ArgumentException
        /// given empty namespace.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructor3EmptyNamespace()
        {
            new MimeMatcher(" ", "matcher.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given non-existing prefix.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructor3NotExist()
        {
            new MimeMatcher(ValidNamespace, "notexists.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given non-existing empty prefix.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructor3NotExistEmpty()
        {
            new MimeMatcher(ValidNamespace, "");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given non-existing empty-space prefix.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructor3NotExistEmptySpace()
        {
            new MimeMatcher(ValidNamespace, " ");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given no list.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructor3NoList()
        {
            new MimeMatcher(ValidNamespace, "matcher_invalid.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given empty string file type name.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructor3EmptyString()
        {
            new MimeMatcher(ValidNamespace, "matcher_empty.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given list containing duplicate element.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructor3DuplicateElement()
        {
            new MimeMatcher(ValidNamespace, "matcher_duplicate.");
        }

        /// <summary>
        /// Tests Match() throws ArgumentNullException
        /// given null FileInfo.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestMatchNull()
        {
            matcher.Match(null);
        }
    }
}
