/*
 * TestCStyleLineCounter.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using System.Collections;
using System.IO;
using NUnit.Framework;
using TopCoder.File.Statistics.Matcher;
using TopCoder.File.Statistics.Tests;

namespace TopCoder.File.Statistics.Processor.LineCounter.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="CStyleLineCounter"/> class.
    /// </summary>
    [TestFixture]
    public class TestCStyleLineCounter : TestHelper
    {
        /// <summary>
        /// The <see cref="CStyleLineCounter"/> instance to test.
        /// Initialized in SetUp().
        /// </summary>
        private IFileProcessor processor;

        /// <summary>
        /// Initializes the <see cref="CStyleLineCounter"/> instance to test.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            // load test_files/Processor_Base.xml
            // used for constructor with nameSpace and prefix argument
            base.LoadConfigFile(@"..\..\test_files\Processor_Base.xml",
                @"..\..\test_files\CStyleLineCounter.xml");

            // initialize with ExtensionMatcher
            processor = new CStyleLineCounter(ValidNamespace, "processor1.");
        }

        /// <summary>
        /// Tests the static properties.
        /// </summary>
        [Test]
        public void TestStaticProperties()
        {
            Assert.AreEqual("Line Count", CStyleLineCounter.Name,
                "The Name member is wrong");
            Assert.AreEqual("line", CStyleLineCounter.DisplayName,
                "The DisplayName member is wrong");
            Assert.AreEqual("support_at_literal", CStyleLineCounter.SupportAtLiteralPropertyKey,
                "The support_at_literal property key is wrong");
        }

        /// <summary>
        /// Tests SupportedMetrics property getter.
        /// </summary>
        [Test]
        public void TestSupportedMetrics()
        {
            // 1 metric is supported
            IList metric = processor.SupportedMetrics;
            Assert.AreEqual(1, metric.Count, "1 metric is supported");

            // assert the metric
            Metric m = metric[0] as Metric;
            Assert.AreEqual(CStyleLineCounter.Name, m.Name,
                "The metric name is wrong");
            Assert.AreEqual(CStyleLineCounter.DisplayName, m.DisplayName,
                "The metric DisplayName is wrong");
        }

        /// <summary>
        /// Tests constructor(string, string) loads alias and file matcher correcly.
        /// </summary>
        [Test]
        public void TestConstructor()
        {
            Assert.AreEqual("Processor1 alias", processor.Alias,
                "The processor alias is not correctly loaded");
            Assert.IsTrue(processor.FileMatcher is ExtensionMatcher,
                "The file matcher is not correctly loaded");

            // assert the matcher can match only txt and sql
            Assert.IsTrue(processor.FileMatcher.Match(new FileInfo("test.txt")),
                "txt should match");
            Assert.IsTrue(processor.FileMatcher.Match(new FileInfo("test.sql")),
                "sql should match");
            Assert.IsFalse(processor.FileMatcher.Match(new FileInfo("test.bin")),
                "bin should not match");
        }

        /// <summary>
        /// Tests constructor(string, string) loads alias and AnyFileMatcher
        /// if no matcher is specified.
        /// </summary>
        [Test]
        public void TestConstructorAnyFileMatcher()
        {
            // use the nomatcher prefix
            processor = new CStyleLineCounter(ValidNamespace, "nomatcher.");

            // assert the alias and AnyFileMatcher
            Assert.AreEqual("Processor2 with AnyFileMatcher", processor.Alias,
                "The processor alias is not correctly loaded");
            Assert.IsTrue(processor.FileMatcher is AnyFileMatcher,
                "AnyFileMatcher should be loaded");

            // assert the matcher can match all
            Assert.IsTrue(processor.FileMatcher.Match(new FileInfo("test.txt")),
                "txt should match");
            Assert.IsTrue(processor.FileMatcher.Match(new FileInfo("test.sql")),
                "sql should match");
            Assert.IsTrue(processor.FileMatcher.Match(new FileInfo("test.bin")),
                "bin should not match");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ArgumentNullException
        /// given null namespace.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructorNullNamespace()
        {
            new CStyleLineCounter(null, "processor1.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ArgumentNullException
        /// given null prefix.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructorNullPrefix()
        {
            new CStyleLineCounter(ValidNamespace, (string) null);
        }

        /// <summary>
        /// Tests constructor(string, string) throws ArgumentException
        /// given empty namespace.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructorEmptyNamespace()
        {
            new CStyleLineCounter(" ", "processor1.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given non-existing prefix.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructorNotExist()
        {
            new CStyleLineCounter(ValidNamespace, "notexists.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given non-existing empty prefix.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructorNotExistEmpty()
        {
            new CStyleLineCounter(ValidNamespace, "");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given non-existing empty-space prefix.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructorNotExistEmptySpace()
        {
            new CStyleLineCounter(ValidNamespace, " ");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given no alias.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructorNoAlias()
        {
            new CStyleLineCounter(ValidNamespace, "noalias.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given empty alias.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructorEmptyAlias()
        {
            new CStyleLineCounter(ValidNamespace, "emptyalias.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given matcher that doesn't exist.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructorNotExistMatcher()
        {
            new CStyleLineCounter(ValidNamespace, "invalidmatcher.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given matcher that is not IFileMatcher.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructorInvalidMatcher()
        {
            new CStyleLineCounter(ValidNamespace, "invalidmatcher2.");
        }

        /// <summary>
        /// Tests constructor(string, IFileMatcher) accuracy.
        /// </summary>
        [Test]
        public void TestConstructor2()
        {
            // instantiate a processor
            processor = new CStyleLineCounter("the alias", new ExtensionMatcher("txt"));

            // assert
            Assert.AreEqual("the alias", processor.Alias,
                "The processor alias is not correctly initialized");
            Assert.IsTrue(processor.FileMatcher is ExtensionMatcher,
                "The file matcher is not correctly initialized");

            // assert the matcher can match only txt
            Assert.IsTrue(processor.FileMatcher.Match(new FileInfo("test.txt")),
                "txt should match");
            Assert.IsFalse(processor.FileMatcher.Match(new FileInfo("test.sql")),
                "sql should match");
            Assert.IsFalse(processor.FileMatcher.Match(new FileInfo("test.bin")),
                "bin should not match");
        }

        /// <summary>
        /// Tests constructor(string, IFileMatcher) throws ArgumentNullException
        /// given null alias name.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructor2NullAlias()
        {
            new CStyleLineCounter(null, new AnyFileMatcher());
        }

        /// <summary>
        /// Tests constructor(string, IFileMatcher) throws ArgumentNullException
        /// given null file matcher.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructor2NullMatcher()
        {
            new CStyleLineCounter("alias", (IFileMatcher) null);
        }

        /// <summary>
        /// Tests constructor(string, IFileMatcher) throws ArgumentException
        /// given empty alias.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructor2EmptyAlias()
        {
            new CStyleLineCounter(" ", new AnyFileMatcher());
        }

        /// <summary>
        /// Tests constructor(string, IFileMatcher, bool) throws ArgumentNullException
        /// given null alias name.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructor3NullAlias()
        {
            new CStyleLineCounter(null, new AnyFileMatcher(), true);
        }
        
        /// <summary>
        /// Tests constructor(string, IFileMatcher, bool) throws ArgumentNullException
        /// given null file matcher.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructor3NullMatcher()
        {
            new CStyleLineCounter("alias", (IFileMatcher) null, true);
        }

        /// <summary>
        /// Tests constructor(string, IFileMatcher, bool) throws ArgumentException
        /// given empty alias name.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructor3EmptyAlias()
        {
            new CStyleLineCounter(" ", new AnyFileMatcher(), true);
        }


        /// <summary>
        /// Tests Process with a sample text file.
        /// </summary>
        [Test]
        public void TestProcess()
        {
            StatsResult result = processor.Process(new FileInfo(SampleText));

            // the sample file contains 4 lines of code
            Assert.AreEqual(1, result.Metrics.Count, "1 metric is returned");
            Assert.AreEqual(0, result[new Metric(CStyleLineCounter.Name, "N/A", "N/A")],
                "0 code lines is expected");
        }

        /// <summary>
        /// Tests Process with an empty file.
        /// </summary>
        [Test]
        public void TestProcessEmpty()
        {
            StatsResult result = processor.Process(new FileInfo(@"..\..\test_files\Matcher_FileCategory_Empty"));

            // the sample file contains no line
            Assert.AreEqual(1, result.Metrics.Count, "1 metric is returned");
            Assert.AreEqual(0, result[new Metric(CStyleLineCounter.Name, "N/A", "N/A")],
                "0 lines of text is expected");
        }

        /// <summary>
        /// Tests Process with a simple C file.
        /// </summary>
        [Test]
        public void TestProcessSimple()
        {
            StatsResult result = processor.Process(new FileInfo(@"..\..\test_files\test\simple.c"));

            // the sample file contains 2 code line2
            Assert.AreEqual(1, result.Metrics.Count, "1 metric is returned");
            Assert.AreEqual(4, result[new Metric(CStyleLineCounter.Name, "N/A", "N/A")],
                "4 code lines is expected");
        }

        /// <summary>
        /// Tests Process with a sample C source code file containing comments only.
        /// </summary>
        [Test]
        public void TestProcessComment()
        {
            StatsResult result = processor.Process(new FileInfo(@"..\..\test_files\test\comment.c"));

            // the sample file contains no line of code
            Assert.AreEqual(1, result.Metrics.Count, "1 metric is returned");
            Assert.AreEqual(0, result[new Metric(CStyleLineCounter.Name, "N/A", "N/A")],
                "There're only comments in the file");
        }

        /// <summary>
        /// Tests Process with a sample C source code file containing literals only.
        /// </summary>
        [Test]
        public void TestProcessLiteral()
        {
            StatsResult result = processor.Process(new FileInfo(@"..\..\test_files\test\literal.c"));

            // the sample file contains no line of code
            Assert.AreEqual(1, result.Metrics.Count, "1 metric is returned");
            Assert.AreEqual(0, result[new Metric(CStyleLineCounter.Name, "N/A", "N/A")],
                "There're only literals in the file");
        }

        /// <summary>
        /// Tests Process with a sample C source code file containing complex codes.
        /// </summary>
        [Test]
        public void TestProcessC()
        {
            StatsResult result = processor.Process(new FileInfo(@"..\..\test_files\test\sample.c"));

            // the sample file contains 12 lines of code
            Assert.AreEqual(1, result.Metrics.Count, "1 metric is returned");
            Assert.AreEqual(22, result[new Metric(CStyleLineCounter.Name, "N/A", "N/A")],
                "22 code lines is expected");
        }

        /// <summary>
        /// Tests Process with a sample C# source code file containing C# string literal.
        /// </summary>
        public void TestProcessCSharpLiteral()
        {
            processor = new CStyleLineCounter("cs", new AnyFileMatcher(), true);
            StatsResult result = processor.Process(new FileInfo(@"..\..\test_files\test2\literal.cs"));

            // the sample file contains only literals (no line of code)
            Assert.AreEqual(1, result.Metrics.Count, "1 metric is returned");
            Assert.AreEqual(0, result[new Metric(CStyleLineCounter.Name, "N/A", "N/A")],
                "0 code line is expected");
        }

        /// <summary>
        /// Tests Process with a sample C# source code file containing C# string literal
        /// and some sample codes.
        /// </summary>
        public void TestProcessCSharpSample()
        {
            processor = new CStyleLineCounter("cs", new AnyFileMatcher(), true);
            StatsResult result = processor.Process(new FileInfo(@"..\..\test_files\test2\sample.cs"));

            // the sample file contains only literals (no line of code)
            Assert.AreEqual(1, result.Metrics.Count, "1 metric is returned");
            Assert.AreEqual(10, result[new Metric(CStyleLineCounter.Name, "N/A", "N/A")],
                "10 code lines are expected");
        }
        
        /// <summary>
        /// Tests Process using configuration file that set support_at_literal to true (case-insensitively).
        /// </summary>
        [Test]
        public void TestProcessAtTrue()
        {
            processor = new CStyleLineCounter("TopCoder.File.Statistics.CStyleLineCounter", "supportTrue.");
            StatsResult result = processor.Process(new FileInfo(@"..\..\test_files\test2\sample.cs"));

            // the sample file contains only literals (no line of code)
            Assert.AreEqual(1, result.Metrics.Count, "1 metric is returned");
            Assert.AreEqual(10, result[new Metric(CStyleLineCounter.Name, "N/A", "N/A")],
                "10 code lines are expected");
        }

        /// <summary>
        /// Tests Process using configuration file that set support_at_literal to yes (case-insensitively).
        /// </summary>
        [Test]
        public void TestProcessAtYes()
        {
            processor = new CStyleLineCounter("TopCoder.File.Statistics.CStyleLineCounter", "supportYes.");
            StatsResult result = processor.Process(new FileInfo(@"..\..\test_files\test2\sample.cs"));

            // the sample file contains only literals (no line of code)
            Assert.AreEqual(1, result.Metrics.Count, "1 metric is returned");
            Assert.AreEqual(10, result[new Metric(CStyleLineCounter.Name, "N/A", "N/A")],
                "10 code lines are expected");
        }
        
        /// <summary>
        /// Tests Process using configuration file that set support_at_literal to on (case-insensitively).
        /// </summary>
        [Test]
        public void TestProcessAtOn()
        {
            processor = new CStyleLineCounter("TopCoder.File.Statistics.CStyleLineCounter", "supportOn.");
            StatsResult result = processor.Process(new FileInfo(@"..\..\test_files\test2\sample.cs"));

            // the sample file contains only literals (no line of code)
            Assert.AreEqual(1, result.Metrics.Count, "1 metric is returned");
            Assert.AreEqual(10, result[new Metric(CStyleLineCounter.Name, "N/A", "N/A")],
                "10 code lines are expected");
        }

        /// <summary>
        /// Tests Process throws ArgumentNullException
        /// given null argument.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestProcessNull()
        {
            processor.Process(null);
        }

        /// <summary>
        /// Tests Process() throws IOException
        /// given a non-existing file.
        /// </summary>
        [Test]
        public void TestProcessNotExist()
        {
            try
            {
                // the file doesn't exist
                // this throws FileNotFoundException which is a subclass of IOException
                processor.Process(new FileInfo(@"..\..\test_files\notexists.txt"));
                Assert.Fail("IOException expected");
            }
            catch (IOException)
            {
                // ok
            }
        }

        /// <summary>
        /// Tests Process() throws IOException
        /// given an existing directory.
        /// </summary>
        [Test]
        [ExpectedException(typeof(IOException))]
        public void TestMatchDirectory()
        {
            // an existing directory
            processor.Process(new FileInfo(@"..\..\test_files"));
        }
        
        /// <summary>
        /// Tests ProposeOptions() return a single ArgumentOption.
        /// </summary>
        [Test]
        public void TestProposeOptions()
        {
            IList options = ((IArgumentHandler) processor).ProposeOptions();

            // a single ArgumentOption
            Assert.AreEqual(1, options.Count, "1 element is returned");
            Assert.IsTrue(options[0] is ArgumentOption, "ArgumentOption is returned");
            
            ArgumentOption option = (ArgumentOption) options[0];
            Assert.AreEqual(CStyleLineCounter.SupportAtLiteralPropertyKey, option.SwitchName,
                "The switch name is incorrect");
            Assert.IsTrue(option.HelpMessage.Length > 0, "There should be a help message");
            Assert.IsFalse(option.IsRequired, "This switch is optional");
            Assert.IsNull(option.Validator, "No validator for this switch");
        }
        
        /// <summary>
        /// Tests HandleArguments() with empty value proceeds in default mode.
        /// </summary>
        [Test]
        public void TestHandleArguments()
        {
            ((IArgumentHandler) processor).HandleArguments(new Hashtable());
            
            StatsResult result = processor.Process(new FileInfo(@"..\..\test_files\test2\sample.cs"));
            
            // the sample file contains 12 lines
            Assert.AreEqual(1, result.Metrics.Count, "1 metric is returned");
            Assert.AreEqual(10, result[new Metric(TextLineCounter.Name, "N/A", "N/A")],
                "10 lines of code is expected");
        }
        
        /// <summary>
        /// Tests HandleArguments() with ignore_blank_line switch with some value,
        /// and it should support C# at literal.
        /// </summary>
        [Test]
        public void TestHandleArgumentsSupport()
        {
            IDictionary args = new Hashtable();
            args.Add(TextLineCounter.IgnoreBlankLinePropertyKey, " ");
            ((IArgumentHandler) processor).HandleArguments(args);
            
            StatsResult result = processor.Process(new FileInfo(@"..\..\test_files\test2\sample.cs"));
            
            // the sample file contains 10 lines of codes
            Assert.AreEqual(1, result.Metrics.Count, "1 metric is returned");
            Assert.AreEqual(10, result[new Metric(TextLineCounter.Name, "N/A", "N/A")],
                "10 lines of code is expected");
        }
        
        /// <summary>
        /// Tests HandleArguments() throws ArgumentNullException
        /// given null argument.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestHandleArgumentsNull()
        {
            ((IArgumentHandler) processor).HandleArguments(null);
        }
        
        /// <summary>
        /// Tests HandleArguments() throws ArgumentException
        /// given non-string value.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestHandleArgumentsNonString()
        {
            IDictionary args = new Hashtable();
            args.Add(CStyleLineCounter.SupportAtLiteralPropertyKey, 123);
            ((IArgumentHandler) processor).HandleArguments(args);
        }
    }
}
