/*
 * TestHelper.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using System.IO;
using System.Text;
using System.Xml;
using NUnit.Framework;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics.Tests
{
    /// <summary>
    /// Common values and functionality for the various unit tests.
    /// </summary>
    public class TestHelper
    {
        /// <summary>
        /// A valid sample configuration file to use.
        /// </summary>
        public const string ValidConfig = @"..\..\test_files\FileStatistics.xml";

        /// <summary>
        /// A sample configuration for MagicNumber.
        /// </summary>
        public const string MagicNumberConfig = @"..\..\test_files\MagicNumber.xml";

        /// <summary>
        /// A valid namespace in the sample configuration file.
        /// </summary>
        public const string ValidNamespace = "TopCoder.File.Statistics";

        /// <summary>
        /// A sample text file for various testing.
        /// </summary>
        public const string SampleText = @"..\..\test_files\test\test.txt";

        /// <summary>
        /// Since ConfigManager is of a singleton design pattern,
        /// the instance can't change during our tests,
        /// so we can just get it once.
        /// </summary>
        protected ConfigManager manager = ConfigManager.GetInstance();

        /// <summary>
        /// Loads new configuration files into the ConfigManager instance.
        /// Clears out old values first.
        /// </summary>
        ///
        /// <param name="configFiles">
        /// Configuration files to load.
        /// </param>
        public void LoadConfigFile(params string[] configFiles)
        {
            manager.Clear(false);
            foreach (string configFile in configFiles)
            {
                manager.LoadFile(configFile);
            }
        }

        /// <summary>
        /// Reads the content of a fileName to a string.
        /// </summary>
        ///
        /// <param name="fileName">
        /// The file name whose content to read.
        /// </param>
        /// <param name="trim">
        /// Removes all white spaces character (including new line) at the beginning
        /// and end of every line in the file content.
        /// </param>
        ///
        /// <returns>
        /// The file content as a <see cref="string"/> or
        /// an empty string if the file doesn't exist or is empty.
        /// </returns>
        public string ReadContent(string fileName, bool trim)
        {
            StringBuilder content = new StringBuilder();

            if (System.IO.File.Exists(fileName))
            {
                // read the file content
                using (StreamReader reader = new StreamReader(fileName))
                {
                    string line;
                    while ( (line = reader.ReadLine()) != null )
                    {
                        // trim if asked
                        content.Append(trim ? line.Trim() : line);
                        if (!trim)
                        {
                            content.Append(Environment.NewLine);
                        }
                    }
                }
            }

            return content.ToString();
        }

        /// <summary>
        /// Asserts that the xml value at the xpath is as expected.
        /// </summary>
        ///
        /// <remarks>
        /// This is used mostly to test that the generated xml report
        /// has the correct value.
        /// </remarks>
        ///
        /// <param name="xml">
        /// The xml string.
        /// </param>
        /// <param name="xpath">
        /// The xpath to extract value.
        /// </param>
        /// <param name="expected">
        /// The expected value (can be null if no value is expected).
        /// </param>
        public void AssertXml(string xml, string xpath, string expected)
        {
            // load into DOM
            XmlDocument doc = new XmlDocument();
            doc.LoadXml(xml);

            // get the xml value at xpath (could be null)
            XmlNode match = doc.SelectSingleNode(xpath);
            string val = null;
            if (match != null)
            {
                val = match.InnerText;
            }

            if (expected == null)
            {
                Assert.IsNull(val, string.Format(
                    "Expected null for {0} in xml {1}", xpath, PrintXml(doc)));
            }
            else
            {
                Assert.IsNotNull(val, string.Format(
                    "Null is not expected for {0} in xml {1}", xpath, PrintXml(doc)));
                Assert.AreEqual(expected, val, string.Format(
                    "The value at {0} is not as expected in xml {1}", xpath, PrintXml(doc)));
            }
        }

        /// <summary>
        /// Asserts that the xml at the xpath has a specified number of child nodes/elements.
        /// </summary>
        ///
        /// <remarks>
        /// This is used to test that the generated xml report
        /// has the correct number of children.
        /// </remarks>
        ///
        /// <param name="xml">
        /// The xml string.
        /// </param>
        /// <param name="xpath">
        /// The xpath whose child count to check.
        /// </param>
        /// <param name="expected">
        /// The expected number of children.
        /// </param>
        public void AssertXmlChildCount(string xml, string xpath, int expected)
        {
            // load into DOM
            XmlDocument doc = new XmlDocument();
            doc.LoadXml(xml);

            // get the xml value at xpath (could be null)
            XmlNodeList children = doc.SelectNodes(xpath);
            Assert.AreEqual(expected, children.Count, string.Format(
                "Expected {0} children for xpath {1} in xml {2}", expected, xpath, PrintXml(doc)));
        }

        /// <summary>
        /// Prints pretty format of xml.
        /// </summary>
        ///
        /// <param name="xml">
        /// The xml document to print.
        /// </param>
        ///
        /// <returns>
        /// A pretty (indented) version of <paramref name="xml"/>.
        /// </returns>
        public string PrintXml(XmlDocument xml)
        {
            //Create an XmlTextWriter which outputs to a StringWriter
            StringWriter stringWriter = new StringWriter();
            XmlTextWriter xmlWriter = new XmlTextWriter(stringWriter);
            // use pretty printing
            xmlWriter.Formatting = Formatting.Indented;
            xmlWriter.Indentation = 4;

            // save to the writer
            xml.WriteTo(xmlWriter);

            // return the string result
            return stringWriter.ToString();
        }
    }
}
